import click
import pudb
from pfmongo import driver
from argparse import Namespace
from pfmongo import env
from pfmisc import Colors as C
from pfmongo.config import settings
from pfmongo.commands.clop import connect
from pfmongo.models import responseModel
from typing import cast
import copy
import asyncio

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
PL = C.PURPLE
YL = C.YELLOW


def options_add(id: str, options: Namespace) -> Namespace:
    localoptions: Namespace = copy.deepcopy(options)
    localoptions.do = "deleteDocument"
    localoptions.argument = id
    localoptions.beQuiet = True
    return localoptions


def env_check(options: Namespace) -> int:
    if env.env_failCheck(options):
        return 100
    return 0


def run_check(
    failData: int, message: str = "early failure"
) -> responseModel.mongodbResponse:
    retm: responseModel.mongodbResponse = responseModel.mongodbResponse()
    if failData:
        retm.message = f" {message} (code {failData}) occurred in document delete"
    else:
        retm.status = True
        retm.message = "Initial run_check in document delete ok"
    return retm


async def delete_do(options: Namespace) -> responseModel.mongodbResponse:
    failEnv: int = 0
    if failEnv := env_check(options):
        return run_check(failEnv, "failure in document del setup")
    delResp: responseModel.mongodbResponse = responseModel.mongodbResponse()
    if not (
        delResp := await driver.run_modelReturn(
            await connect.baseCollection_getAndConnect(options)
        )
    ).status:
        pass
        # return delResp
    print(delResp.message)
    if not settings.appsettings.donotFlatten:
        delResp = await driver.run_modelReturn(
            await connect.shadowCollection_getAndConnect(options)
        )
    return delResp


async def deleteDo_asInt(options: Namespace) -> int:
    delResp: responseModel.mongodbResponse = await delete_do(options)
    docDelUse: responseModel.DocumentDeleteUsage = responseModel.DocumentDeleteUsage()
    docDelUse.status = delResp.status
    return env.response_exitCode(docDelUse)


async def deleteDo_asModel(options: Namespace) -> responseModel.mongodbResponse:
    return await delete_do(options)


def sync_deleteDo_asInt(options: Namespace) -> int:
    return asyncio.run(deleteDo_asInt(options))


def sync_deleteDo_asModel(options: Namespace) -> responseModel.mongodbResponse:
    return asyncio.run(deleteDo_asModel(options))


@click.command(
    cls=env.CustomCommand,
    help=f"""
remove a {PL}document{NC} from a collection

SYNOPSIS
{CY}delete {GR}--id {YL}<id>{NC}

DESC
This subcommand removes a {PL}document{NC} identified by {YL}<id>{NC} from a collection.

The "location" is defined by the core parameters, 'useDB' and 'useCollection'
which are typically defined in the CLI, in the system environment, or in the
session state.

Use with care. No confirmation is asked!

""",
)
@click.option(
    "--id", type=str, help="Delete the document with the passed 'id'", default=""
)
@click.pass_context
def delete(ctx: click.Context, id: str = "") -> int:
    # pudb.set_trace()
    return sync_deleteDo_asInt(options_add(id, ctx.obj["options"]))



import click
import pudb
from pfmongo import driver, env
from argparse import Namespace
from pfmongo.models import responseModel
from pfmisc import Colors as C
import copy
import asyncio

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
PL = C.PURPLE
YL = C.YELLOW


def options_add(id: str, options: Namespace) -> Namespace:
    localoptions: Namespace = copy.deepcopy(options)
    localoptions.do = "getDocument"
    localoptions.argument = id
    return localoptions


def get_envFailCheck(options: Namespace) -> int:
    if env.env_failCheck(options):
        return 100
    return 0


async def documentGet_asInt(options) -> int:
    fail: int = 0
    if fail := get_envFailCheck(options):
        return fail
    return await driver.run_intReturn(options)


async def documentGet_asModel(options) -> responseModel.mongodbResponse:
    model: responseModel.mongodbResponse = responseModel.mongodbResponse()
    if get_envFailCheck(options):
        model.message = "env failure"
        return model
    return await driver.run_modelReturn(options)


def sync_documentGet_asInt(options: Namespace) -> int:
    return asyncio.run(documentGet_asInt(options))


def sync_documentGet_asModel(options: Namespace) -> responseModel.mongodbResponse:
    return asyncio.run(documentGet_asModel(options))


@click.command(
    cls=env.CustomCommand,
    help=f"""
read and print {PL}document{NC} from a collection

SYNOPSIS
{CY}get {YL}--id <id>{NC}

DESC
This subcommand gets a document with passed 'id' from a collection and
prints it to the console. Use shell redirection to save to disk.

The "location" is defined by the core parameters, 'useDB' and 'useCollection'
which are typically defined in the CLI, in the system environment, or in the
session state.

""",
)
@click.option("--id", type=str, help="the document 'id' to get", default="")
@click.pass_context
def get(ctx: click.Context, id: str = "") -> int:
    # pudb.set_trace()
    return sync_documentGet_asInt(options_add(id, ctx.obj["options"]))



str_description = """

    The data models/schemas for the PACS QR collection.

"""

from pydantic import BaseModel, Field
from typing import Optional, List, Dict
from datetime import datetime
from enum import Enum
from pathlib import Path
from pfmongo.config import settings


class stateResponse(BaseModel):
    database: str = ""
    collection: str = ""
    app: settings.App = settings.appsettings
    mongo: settings.Mongo = settings.mongosettings


class mongodbResponse(BaseModel):
    """The response model from the mongodb server"""

    status: bool = False
    message: str = ""
    response: dict = {}
    exitCode: int = 0


# Connection status returns


class databaseConnectStatus(BaseModel):
    status: bool = False
    connected: bool = False
    existsAlready: bool = False
    error: str = ""


class collectionConnectStatus(BaseModel):
    status: bool = False
    connected: bool = False
    existsAlready: bool = False
    elements: int = 0
    error: str = ""


# API connection returns


class databaseDesc(BaseModel):
    status: bool = False
    otype: str = "database"
    host: str = ""
    port: int = -1
    name: str = ""
    info: databaseConnectStatus = databaseConnectStatus()


class collectionDesc(BaseModel):
    status: bool = False
    database: databaseDesc = databaseDesc()
    otype: str = "collection"
    databaseName: str = ""
    name: str = ""
    fullName: str = ""
    info: collectionConnectStatus = collectionConnectStatus()


# API usage returns


class showAllDBusage(BaseModel):
    """response for getting a list of all databases in a mongoDB"""

    status: bool = False
    otype: str = "accessing database names"
    databaseNames: list = []
    info: databaseConnectStatus = databaseConnectStatus()


class showAllcollectionsUsage(BaseModel):
    """response for getting a list of all collections in a database"""

    status: bool = False
    otype: str = "accessing collection names"
    collectionNames: list = []
    info: databaseConnectStatus = databaseConnectStatus()


class databaseNamesUsage(BaseModel):
    status: bool = False
    otype: str = "accessing database names"
    databaseNames: list = []
    info: databaseConnectStatus = databaseConnectStatus()


class collectionNamesUsage(BaseModel):
    status: bool = False
    otype: str = "accessing collection names"
    collectionNames: list = []
    info: databaseConnectStatus = databaseConnectStatus()


class DocumentAddUsage(BaseModel):
    """response for adding a document to a collection"""

    status: bool = False
    otype: str = "adding document"
    documentName: str = ""
    document: dict = {}
    resp: dict = {}
    collection: collectionDesc = collectionDesc()


class DocumentDeleteUsage(BaseModel):
    """response for deleting a document from a collection"""

    status: bool = False
    otype: str = "deleting document"
    documentName: str = ""
    document: dict = {}
    resp: dict = {}
    collection: collectionDesc = collectionDesc()


class CollectionDeleteUsage(BaseModel):
    """response for deleting a collection from a database"""

    status: bool = False
    otype: str = "deleting collection"
    collectionName: str = ""
    resp: dict = {}
    collection: collectionDesc = collectionDesc()


class dbDeleteUsage(BaseModel):
    """response for deleting a database from a mongo server"""

    status: bool = False
    otype: str = "deleting collection"
    dbName: str = ""
    resp: dict = {}
    db: databaseDesc = databaseDesc()


class DocumentListUsage(BaseModel):
    """response for listing all documents from a collection"""

    status: bool = False
    otype: str = "listing all documents"
    documentField: str = ""
    documentList: list = []
    resp: dict = {}
    collection: collectionDesc = collectionDesc()


class DocumentSearchUsage(DocumentListUsage):
    """response for listing all documents from a collection"""

    otype: str = "searching all documents"


class DocumentGetUsage(BaseModel):
    """response for getting adding a document from a collection"""

    status: bool = False
    otype: str = "getting document"
    documentName: str = ""
    document: dict = {}
    resp: dict = {}
    collection: collectionDesc = collectionDesc()



from importlib.metadata import Distribution, PackageNotFoundError

__version__ = "0.9.900"

__pkg: Distribution
__version__: str
__pkg_name__: str

try:
    __pkg = Distribution.from_name(__package__)
    __version__ = __pkg.version
    __pkg_name__ = __pkg.metadata["Name"]
except PackageNotFoundError:
    __pkg = None
    __pkg_name__ = "pfmongo"



import click
import pudb
from pfmongo import driver, env
import pfmongo.pfmongo
from argparse import Namespace
import json
from pfmisc import Colors as C, switch
from pfmongo.config import settings
from pfmongo.models import responseModel
from typing import Tuple, cast, Callable, Coroutine, Awaitable
from pfmongo.commands.clop import connect
from pfmongo.models.dataModel import messageType
import copy
import asyncio

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
PL = C.PURPLE
YL = C.YELLOW


def options_add(input: str, id: str, options: Namespace) -> Namespace:
    """
    Add options for a document addition operation.

    :param input: Input text (file path or JSON string) to be added.
    :param id: Unique identifier for the document.
    :param options: Namespace object with the current options.
    :return: Updated options Namespace.
    """
    localoptions: Namespace = copy.deepcopy(options)
    localoptions.do = "addDocument"
    localoptions.argument = {"file": input, "id": id}
    return localoptions


def flatten_dict(data: dict, parent_key: str = "", sep: str = "/") -> dict:
    """
    Flatten a nested dictionary into a single level using a separator.

    :param data: Dictionary to flatten.
    :param parent_key: Key of the parent element (used for recursion).
    :param sep: Separator to use for keys in the flattened dictionary.
    :return: Flattened dictionary.
    """
    flattened: dict = {}
    for k, v in data.items():
        new_key = f"{parent_key}{sep}{k}" if parent_key else k
        if isinstance(v, dict):
            flattened.update(flatten_dict(v, new_key, sep=sep))
        elif isinstance(v, list):
            for i, item in enumerate(v):
                list_key = f"{new_key}{sep}{i}"
                if isinstance(item, (dict, list)):
                    flattened.update(
                        flatten_dict({str(i): item}, parent_key=list_key, sep=sep)
                    )
                else:
                    flattened[list_key] = item
        else:
            flattened[new_key] = v
    return flattened


def env_OK(options: Namespace, d_doc: dict) -> bool | dict:
    """
    Check the environment and document status.

    :param options: Namespace object with the current options.
    :param d_doc: Document to validate.
    :return: False if the environment or document is invalid, otherwise the document data.
    """
    envFailure: int = env.env_failCheck(options)
    if envFailure:
        return False
    if not d_doc["status"]:
        return not bool(env.complain(d_doc["data"], 1, messageType.ERROR))
    if "data" in d_doc:
        return d_doc["data"]
    else:
        return False


def input_intoDictRead(input: str) -> dict[str, dict | str]:
    """
    Attempt to read input as either a file or a JSON string and parse it into a dictionary.

    :param input: Input text, either a file path or a JSON string.
    :return: A dictionary with the status and parsed data or error details.
    """
    d_json: dict = {"status": False, "input": input, "data": {}}

    # Attempt to open the input as a file
    try:
        with open(input, "r") as f:
            d_json["data"] = json.load(f)
            d_json["status"] = True
            return d_json
    except FileNotFoundError:
        pass  # Input is not a valid file; proceed to JSON parsing
    except Exception as e:
        d_json["data"] = f"File read error: {str(e)}"
        return d_json

    # Attempt to parse the input as a JSON string
    try:
        d_json["data"] = json.loads(input)
        d_json["status"] = True
    except Exception as e:
        d_json["data"] = f"JSON parse error: {str(e)}"

    return d_json


async def prepCollection_forDocument(
    options: Namespace,
    connectCollection: Callable[[Namespace], Awaitable[Namespace]],
    document: dict,
) -> Callable[..., Coroutine[None, None, int | responseModel.mongodbResponse]]:
    """
    Prepare a collection for adding a document.

    :param options: Namespace object with the current options.
    :param connectCollection: Callable to connect to the collection.
    :param document: Document to prepare for insertion.
    :return: Callable for the collection operation.
    """
    document["_date"] = pfmongo.pfmongo.timenow()
    document["_owner"] = settings.mongosettings.MD_sessionUser
    document["_size"] = driver.get_size(document)
    collection: Namespace = await connectCollection(options)
    return driver.event_setup(
        driver.settmp(
            options,
            [
                {"collectionName": collection.collectionName},
                {"argument": document},
            ],
        )
    )


async def add_asType(
    document: dict, options: Namespace, modelReturnType: str
) -> int | responseModel.mongodbResponse:
    """
    Add a document to the shadow and primary collections.

    :param document: Document to add.
    :param options: Namespace object with the current options.
    :param modelReturnType: Desired return type ("int" or "model").
    :return: Operation status as an integer or MongoDB response model.
    """
    shadowResp: responseModel.mongodbResponse = responseModel.mongodbResponse()
    if not settings.appsettings.donotFlatten:
        run = await prepCollection_forDocument(
            options, connect.shadowCollection_getAndConnect, flatten_dict(document)
        )
        saveShadowFail: int | responseModel.mongodbResponse = await run(
            printResponse=False, returnType="model"
        )
        shadowResp = cast(responseModel.mongodbResponse, saveShadowFail)
        if not shadowResp.status:
            match modelReturnType:
                case "model":
                    return saveShadowFail
                case "int":
                    return int(shadowResp.exitCode)
    if not options.argument["id"] and shadowResp.status:
        document["_id"] = shadowResp.response["connect"].documentName
    run = await prepCollection_forDocument(
        options, connect.baseCollection_getAndConnect, document
    )
    return await run(printResponse=True, returnType=modelReturnType)


def setup(options: Namespace) -> Tuple[int, dict]:
    """
    Set up the environment and validate the input document.

    :param options: Namespace object with the current options.
    :return: Tuple of setup status and document data.
    """
    d_data: dict = {}
    if env.env_failCheck(options):
        return 100, d_data
    d_dataOK: dict | bool = env_OK(
        options, input_intoDictRead(options.argument["file"])
    )
    if not d_dataOK:
        return 100, d_data
    if not isinstance(d_dataOK, dict):
        return 101, d_data
    d_data = d_dataOK
    if len(options.argument["id"]):
        d_data["_id"] = options.argument["id"]
    return 0, d_data


def earlyFailure(
    failData: Tuple[int, dict], returnType: str = "int"
) -> int | responseModel.mongodbResponse:
    """
    Handle early failures during setup.

    :param failData: Tuple of failure code and data.
    :param returnType: Desired return type ("int" or "model").
    :return: Failure response as an integer or MongoDB response model.
    """
    reti: int = failData[0]
    retm: responseModel.mongodbResponse = responseModel.mongodbResponse()
    retm.message = f"A setup failure of return {reti} occurred"
    match returnType:
        case "int":
            return reti
        case "model":
            return retm
        case _:
            return reti


async def documentAdd_asType(
    options: Namespace, returnType: str = "int"
) -> int | responseModel.mongodbResponse:
    """
    Add a document and return the result in the specified type.

    :param options: Namespace object with the current options.
    :param returnType: Desired return type ("int" or "model").
    :return: Operation status as an integer or MongoDB response model.
    """
    failOrOK: Tuple[int, dict] = (-1, {})
    if (failOrOK := setup(options))[0]:
        return earlyFailure(failOrOK, returnType)
    d_data: dict = failOrOK[1]
    return await add_asType(d_data, options, returnType)


async def documentAdd_asInt(options: Namespace) -> int:
    """
    Add a document and return the result as an integer.

    :param options: Namespace object with the current options.
    :return: Operation status as an integer.
    """
    return cast(int, await documentAdd_asType(options, "int"))


async def documentAdd_asModel(options: Namespace) -> responseModel.mongodbResponse:
    """
    Add a document and return the result as a MongoDB response model.

    :param options: Namespace object with the current options.
    :return: Operation status as a MongoDB response model.
    """
    return cast(
        responseModel.mongodbResponse, await documentAdd_asType(options, "model")
    )


def sync_documentAdd_asInt(options: Namespace) -> int:
    """
    Synchronously add a document and return the result as an integer.

    :param options: Namespace object with the current options.
    :return: Operation status as an integer.
    """
    return asyncio.run(documentAdd_asInt(options))


def sync_documentAdd_asModel(options: Namespace) -> responseModel.mongodbResponse:
    """
    Synchronously add a document and return the result as a MongoDB response model.

    :param options: Namespace object with the current options.
    :return: Operation status as a MongoDB response model.
    """
    return asyncio.run(documentAdd_asModel(options))


@click.command(
    cls=env.CustomCommand,
    help=f"""
add a {PL}document{NC} to a collection

SYNOPSIS
{CY}add {YL}[--input <filename>|<jsonObj>] [--id <value>]{NC}

DESC
This subcommand adds a document to the associated {YL}COLLECTION{NC}.
The document can either be read from a file on the file system or directly
as a JSON serialized string.

A "shadow" document with a flat dataspace is also added to a "shadow"
collection for efficient searching and is kept "in sync" with the original.

The "location" is defined by the core parameters, 'useDB' and 'useCollection'
which are typically defined in the CLI, in the system environment, or in the
session state.
""",
)
@click.option(
    "--input",
    type=str,
    help="Input as a file path or a JSON serialized string.",
)
@click.option(
    "--id",
    type=str,
    help="If specified, set the 'id' in the mongo collection to the passed string.",
    default="",
)
@click.pass_context
def add(ctx: click.Context, input: str, id: str = "") -> int:
    """
    CLI command to add a document to a MongoDB collection.

    :param ctx: Click context object.
    :param input: Input as a file path or a JSON serialized string.
    :param id: Unique identifier for the document.
    :return: Operation status as an integer.
    """
    return sync_documentAdd_asInt(options_add(input, id, ctx.obj["options"]))



from inspect import trace
from pfmongo import pfmongo
from pfmongo.__main__ import main
import os, pudb
from argparse import Namespace
from pfmongo.commands.dbop import connect as database, deleteDB
from pfmongo.commands.clop import connect as collection
from pfmongo.commands.docop import add, get
from pfmongo.commands import smash
from pfmongo.models import responseModel
from pfmongo.config import settings
from pfmongo import driver
import json
import pytest
import asyncio

import tracemalloc

tracemalloc.start()

os.environ["XDG_CONFIG_HOME"] = "/tmp"


@pytest.mark.asyncio
async def DB_connect(DB: str = "testDB") -> int:
    return await database.connectTo_asInt(
        database.options_add(DB, pfmongo.options_initialize())
    )


@pytest.mark.asyncio
async def collection_connect(col: str = "testCollection") -> int:
    return await collection.connectTo_asInt(
        collection.options_add(col, pfmongo.options_initialize())
    )


@pytest.mark.asyncio
async def DB_delete(DB: str = "testDB") -> int:
    return await deleteDB.DBdel_asInt(
        deleteDB.options_add(DB, pfmongo.options_initialize())
    )


@pytest.mark.asyncio
async def smash_do(cmd: str) -> str:
    options: Namespace = pfmongo.options_initialize()
    options.eventLoopDebug = True
    resp: str | bytes = await smash.smash_execute_async(
        smash.command_parse(await smash.command_get(options, noninteractive=cmd))
    )
    if isinstance(resp, str):
        return resp
    if isinstance(resp, bytes):
        return resp.decode()
    return "Invalid response type received"


@pytest.mark.asyncio
async def test_database_showAll() -> None:
    # pudb.set_trace()
    databases: str = await smash_do("database showall")
    assert "admin" in databases


if __name__ == "__main__":
    print("Test document operations")
    pudb.set_trace()
    asyncio.run(test_database_showAll())



"""
Program entry point for the pfmongo CLI.

This module initializes the application, processes arguments using
argparse, and invokes the main Click application defined in app_core.py.
"""

import sys
import re
from argparse import ArgumentParser, Namespace
from typing import Sequence, Optional
from pfmongo.pfmongo import (
    options_initialize,
    parser_setup,
    parser_interpret,
)
from pfmongo import env
from pfmongo.app_core import app
from pfmongo.commands import man


# Global options namespace
options: Namespace = Namespace()


def namespace_pubattribs(space: Namespace) -> list[str]:
    """
    Retrieve public attributes from a Namespace object.

    :param space: The Namespace object to analyze.
    :return: A list of public attribute names.
    """
    attributes: list[str] = re.findall(r"\b([A-Za-z][A-Za-z_0-9]*)=", str(space))
    return attributes


def sysargv_revamp(newarg: list[str]) -> list[str]:
    """
    Revamp sys.argv for Click command-line parsing.

    Replaces the current sys.argv with a new set of arguments
    while preserving the executable name.

    :param newarg: A list of new arguments to set.
    :return: The updated sys.argv list.
    """
    executable: str = sys.argv[0]
    sys.argv = [executable] + newarg
    return sys.argv


def cli_entry_point(argv: Optional[list[str]] = None) -> int:
    """
    Core logic for CLI entry, testable separately from Click.

    This function processes core CLI arguments using argparse,
    checks environmental configurations, and prepares for Click dispatch.

    :param argv: A list of CLI arguments (e.g., `["--man"]`).
    :return: 0 on success, or an error code on failure.
    """
    global options
    argv = argv or sys.argv[1:]
    add_help: bool = False

    # Set up and parse arguments using argparse
    parser: ArgumentParser = parser_setup(
        "A client for interacting with MongoDB.", add_help
    )
    options, extra = parser_interpret(parser, argv)

    # Verify environmental state path
    if not env.env_statePathSet(options):
        return 10

    # This is the result of argparse
    if man.coreOptions_show(options):
        return man.man(options)

    # Update sys.argv for Click
    sysargv_revamp(extra)
    return app()


def main(argv: Optional[list[str]] = None) -> int:
    """
    Entry point for the CLI application.

    This wraps the core CLI logic (`cli_entry_point`) to allow system exit handling
    during normal execution.
    """
    try:
        return cli_entry_point(argv)
    except SystemExit as e:
        return e.code


if __name__ == "__main__":
    sys.exit(main())



from argparse import Namespace
from typing import Any, List, TypedDict, Mapping, Optional
from pydantic import BaseModel, Field

import json
from datetime import datetime
from pathlib import Path

import sys
import shutil
import pudb

import pymongo
from pymongo import MongoClient
from pymongo.database import Database
from pymongo.collection import Collection
from pymongo import results

from pfmongo.config import settings
from pfmongo.models import responseModel
from pfmongo import env

from motor import motor_asyncio as AIO
from motor.core import AgnosticCursor
from bson import ObjectId
import hashlib
import copy
import re


class mongoDB:
    def getDB(self) -> AIO.AsyncIOMotorDatabase:
        return self.DB

    async def database_names_get(self) -> responseModel.showAllDBusage:
        resp: responseModel.showAllDBusage = responseModel.showAllDBusage()
        l_DBs: list = []
        error: str = ""
        connected: bool = True
        try:
            l_DBs = await self.Mongo.list_database_names()
        except Exception as e:
            connected = False
            error = f"{e}"
        resp.info.connected = connected
        resp.status = connected
        resp.info.error = error
        resp.databaseNames = l_DBs
        return resp

    async def collection_names_get(self) -> responseModel.showAllcollectionsUsage:
        collection: responseModel.showAllcollectionsUsage = (
            responseModel.showAllcollectionsUsage()
        )
        collection.collectionNames = await self.DB.list_collection_names()
        collection.status = True
        return collection

    async def connectDB(self, DBname: str) -> dict:
        """
        Connect to the DB called <DBname> (or create the DB if it does
        not exist). Return the DB and a bool exist_already/not_exist
        in a dictionary.

        Args:
            DBname (str): the DB name

        Returns:
            dict[str, bool | Database[Any]]: DB -- the database
                                             bool -- False if DB is not yet created
        """
        dbnames: responseModel.showAllDBusage = await self.database_names_get()
        d_ret: dict = {
            "connected": dbnames.info.connected,
            "existsAlready": True if DBname in dbnames.databaseNames else False,
            "DB": self.Mongo[DBname],
            "error": dbnames.info.error,
        }
        return d_ret

    async def db_delete(
        self, dbDelete: responseModel.dbDeleteUsage
    ) -> responseModel.dbDeleteUsage:
        await self.Mongo.drop_database(dbDelete.dbName)
        dbDelete.status = True
        return dbDelete

    def insert_one_response(self, d_resp: dict) -> dict[bool, dict]:
        d_data: dict = {"status": False, "document": d_resp}
        if "acknowledged" in d_resp:
            d_data["status"] = d_resp["acknowledged"]
        return d_data

    async def insert_one(
        self, collection: responseModel.collectionDesc, **kwargs
    ) -> responseModel.DocumentAddUsage:
        # pudb.set_trace()
        insert: responseModel.DocumentAddUsage = responseModel.DocumentAddUsage()
        d_document: dict = {}
        for k, v in kwargs.items():
            if k == "document":
                d_document = v
        insert.document = d_document
        insert.collection = collection
        ld_collection: list = [
            d for d in self.ld_collection if d["name"] == collection.name
        ]
        for d in ld_collection:
            insert.resp = await d["object"].document_add(d_document)
            insert.status = insert.resp["acknowledged"]
            insert.documentName = insert.resp["inserted_id"]
        return insert

    async def delete_one(
        self, collection: responseModel.collectionDesc, **kwargs
    ) -> responseModel.DocumentDeleteUsage:
        delete: responseModel.DocumentDeleteUsage = responseModel.DocumentDeleteUsage()
        id: str = ""
        for k, v in kwargs.items():
            if k == "document":
                id = v
        delete.documentName = id
        delete.collection = collection
        ld_collection: list = [
            d for d in self.ld_collection if d["name"] == collection.name
        ]
        for d in ld_collection:
            delete.resp = await d["object"].document_delete(id)
            delete.status = (
                delete.resp["acknowledged"] if delete.resp["deleted_count"] else False
            )
        return delete

    async def get_one(
        self, collection: responseModel.collectionDesc, **kwargs
    ) -> responseModel.DocumentGetUsage:
        get: responseModel.DocumentGetUsage = responseModel.DocumentGetUsage()
        id: str = ""
        for k, v in kwargs.items():
            if k == "document":
                id = v
        get.documentName = id
        get.collection = collection
        ld_collection: list = [
            d for d in self.ld_collection if d["name"] == collection.name
        ]
        for d in ld_collection:
            get.resp = await d["object"].document_get(id)
            get.status = get.resp["acknowledged"]
        if get.status:
            get.document = get.resp["document"]
            get.resp["document"] = "Read successfully"
        return get

    async def listDocs(
        self, collection: responseModel.collectionDesc, **kwargs
    ) -> responseModel.DocumentListUsage:
        """Note: no differentiation between failed call and empty list"""
        docs: responseModel.DocumentListUsage = responseModel.DocumentListUsage()
        field: str = ""
        for k, v in kwargs.items():
            if k == "field":
                field = v
        docs.documentField = field
        docs.collection = collection
        ld_collection: list = [
            d for d in self.ld_collection if d["name"] == collection.name
        ]
        for d in ld_collection:
            docs.resp = await d["object"].document_list(field)
            docs.status = docs.resp["acknowledged"]
        if docs.status:
            docs.documentList = docs.resp["find_list"]
            docs.resp["find_list"] = "Listed successfully"
        docs.status = True
        return docs

    async def searchDocs(
        self, collection: responseModel.collectionDesc, **kwargs
    ) -> responseModel.DocumentSearchUsage:
        docs: responseModel.DocumentSearchUsage = responseModel.DocumentSearchUsage()
        field: str = ""
        searchFor: list = []
        for k, v in kwargs.items():
            if k == "searchFor":
                searchFor = v
            if k == "field":
                field = v
        docs.documentField = field
        docs.collection = collection
        ld_collection: list = [
            d for d in self.ld_collection if d["name"] == collection.name
        ]
        for d in ld_collection:
            docs.resp = await d["object"].document_search(searchFor, field)
            docs.status = docs.resp["acknowledged"]
        if docs.status:
            docs.documentList = docs.resp["find_list"]
            docs.resp["find_list"] = "Listed successfully"
        return docs

    def collection_serialize(self) -> responseModel.collectionDesc:
        resp: responseModel.collectionDesc = responseModel.collectionDesc()
        return resp

    async def connectCol(self, name: str) -> dict:
        colObj: mongoCollection = mongoCollection(self)
        return {
            "name": name,
            "collection": await colObj.connect(name),
            "object": colObj,
            "interface": self.Mongo[name],
        }

    async def collection_delete(self, name: str) -> responseModel.CollectionDeleteUsage:
        # pudb.set_trace()
        resp: responseModel.CollectionDeleteUsage = (
            responseModel.CollectionDeleteUsage()
        )
        resp.collectionName = name
        l_names = [x["name"] for x in self.ld_collection]
        if name not in l_names:
            resp.resp["error"] = "collection not found"
        else:
            lookup: dict = {}
            lookupOrNone: dict | None = next(
                (i for i in self.ld_collection if i["name"] == name), None
            )
            if lookupOrNone:
                lookup = lookupOrNone
                resp.collection = lookup["collection"]
                if not resp.collection.info.error:
                    collectionToRemove: AIO.AsyncIOMotorCollection = lookup[
                        "object"
                    ].d_collection["interface"]
                    await collectionToRemove.drop()
                    self.ld_collection.remove(lookup)
                    resp.status = True
        return resp

    async def collection_connect(self, name: str) -> responseModel.collectionDesc:
        resp: responseModel.collectionDesc = responseModel.collectionDesc()
        l_names = [x["name"] for x in self.ld_collection]
        if name not in l_names:
            self.ld_collection.append(await self.connectCol(name))
            resp = self.ld_collection[-1]["collection"]
        else:
            lookup: responseModel.collectionDesc | None = next(
                (i["collection"] for i in self.ld_collection if i["name"] == name), None
            )
            if lookup:
                resp.collectionDesc = lookup
        return resp

    def __init__(self, **kwargs) -> None:
        """
        The constructor for the mongo data base "object". Each "object"
        is connected to a single mongo database, which can contain a variable
        number of collections (described in a list).

        For "simplicity sake" in this formulation, a new object connection
        is created for each "call"/"use" of this module (i.e. connections
        are single-use).

        Be sure to await the "connect" method on this object after instantiation!

        :param DBname: the string name of the data base within Mongo
        :param settingsMongo: a structure containing the Mongo URI and login
        :return: None -- this is a constructor
        """
        settingsMongo: settings.Mongo = settings.Mongo()
        args: Namespace = Namespace()

        for k, v in kwargs.items():
            if k == "settings":
                settingsMongo = v
            if k == "args":
                args = v

        self.Mongo: AIO.AsyncIOMotorClient = AIO.AsyncIOMotorClient(
            settingsMongo.MD_URI,
            username=settingsMongo.MD_username,
            password=settingsMongo.MD_password,
        )
        self.ld_collection: list[dict] = []
        self.args = args

    def database_serialize(self) -> responseModel.databaseDesc:
        resp: responseModel.databaseDesc = responseModel.databaseDesc()
        # pudb.set_trace()
        resp.host = self.DB.client.HOST
        resp.port = self.DB.client.PORT
        resp.name = str(self.DB.name)
        resp.info.connected = self.d_DBref["connected"]
        resp.info.existsAlready = self.d_DBref["existsAlready"]
        resp.info.error = self.d_DBref["error"]
        resp.info.status = resp.info.connected
        resp.status = resp.info.status
        return resp

    async def connect(self, DBname: str) -> responseModel.databaseDesc:
        self.d_DBref: dict = await self.connectDB(DBname)
        # pudb.set_trace()
        self.DB: AIO.AsyncIOMotorDatabase = self.d_DBref["DB"]
        return self.database_serialize()


class mongoCollection:
    async def search_on(self, d_query: dict) -> list:
        l_hits: list = []
        hits: AIO.AsyncIOMotorCursor = self.collection.find(d_query)
        async for hit in hits:
            l_hits.append(hit)
        # pudb.set_trace()
        return l_hits

    def hash_excludeKeyVal(self, d_hash: dict) -> dict:
        d_hashEx: dict = copy.deepcopy(d_hash)
        keysToClear: list[str] = ["_id", "_date", "_size", "_owner"]
        d_cleared: dict[str, str] = {key: "" for key in keysToClear}
        d_hashEx.update(d_cleared)
        return d_hashEx

    def hash_addToDocument(self, d_doc: dict, l_onlyUsekeys: list = []) -> dict:
        d_newDict: dict = copy.deepcopy(d_doc)
        d_toHash: dict = {}
        if l_onlyUsekeys:
            d_toHash = {k: d_doc[k] for k in l_onlyUsekeys}
        else:
            d_toHash = d_doc
        d_toHash = self.hash_excludeKeyVal(d_toHash)
        d_sorted: dict = dict(sorted(d_toHash.items()))
        hash = hashlib.sha256(str(d_sorted).encode()).hexdigest()
        d_newDict["hash"] = hash
        return d_newDict

    async def is_duplicate(self, d_doc: dict) -> bool:
        b_ret: bool = False
        if "hash" not in d_doc.keys():
            return b_ret
        l_hits: list = await self.search_on({"hash": d_doc["hash"]})
        if len(l_hits):
            b_ret = True
        return b_ret

    def dbcol_add(self, d_resp: dict) -> dict:
        d_resp["database"] = self.DB.name
        d_resp["collection"] = self.collection.name
        return d_resp

    async def document_add(self, d_data: dict) -> dict:
        # pudb.set_trace()
        d_resp: dict = self.dbcol_add(
            {"acknowledged": False, "inserted_id": "-1", "error": ""}
        )
        if self.addHashToDocument:
            d_data = self.hash_addToDocument(d_data)
        if await self.is_duplicate(d_data) and not self.allowDuplicates:
            d_resp["error"] = "Duplicate document hash found."
            d_resp["inserted_id"] = d_data["_id"]
        else:
            resp: results.InsertOneResult = results.InsertOneResult(None, False)
            try:
                resp = await self.collection.insert_one(d_data)
            except Exception as e:
                d_resp["error"] = "%s" % e
            d_resp["acknowledged"] = resp.acknowledged
            d_resp["inserted_id"] = str(resp.inserted_id)
        return d_resp

    async def document_delete(self, id: str) -> dict:
        d_resp: dict = {
            "acknowledged": False,
            "deleted_count": -1,
            "database": "",
            "collection": "",
            "error": "",
        }
        query: dict = {"_id": id}
        raw_result: Mapping[str, int] = {"n": 1}
        resp: results.DeleteResult = results.DeleteResult(raw_result, False)
        for _ in range(0, 2):
            try:
                resp = await self.collection.delete_one(query)
            except Exception as e:
                d_resp["error"] = "%s" % e
            d_resp["acknowledged"] = resp.acknowledged
            d_resp["deleted_count"] = resp.deleted_count
            d_resp["database"] = self.DB.name
            d_resp["collection"] = self.collection.name
            if resp.deleted_count:
                break
            try:
                query = {"_id": ObjectId(id)}
            except Exception as e:
                break
        return d_resp

    async def document_get(self, id: str) -> dict:
        # pudb.set_trace()
        d_resp: dict = {
            "acknowledged": False,
            "document": {},
            "database": "",
            "collection": "",
            "error": "",
        }
        query: dict = {"_id": id}
        for _ in range(0, 2):
            document: Optional[dict[str, Any]] = await self.collection.find_one(query)
            if document:
                document["_id"] = str(document["_id"])
                d_resp["acknowledged"] = True
                d_resp["document"] = document
                d_resp["database"] = self.DB.name
                d_resp["collection"] = self.collection.name
                break
            try:
                query = {"_id": ObjectId(id)}
            except Exception as e:
                d_resp["error"] = f"Could not access {'_id'}: {e}"
        return d_resp

    async def document_list(self, field: str) -> dict:
        # pudb.set_trace()
        d_resp: dict = {"acknowledged": False, "find_list": [], "error": ""}
        query: dict = {field: 1}
        cursor: AgnosticCursor = self.collection.find({}, query)
        async for doc in cursor:
            d_resp["find_list"].append(doc[field])
        if len(d_resp["find_list"]):
            d_resp["acknowledged"] = True
        return d_resp

    async def create_text_index(self):
        """Create a text index on all fields in the collection."""
        await self.collection.create_index([("$**", "text")])

    async def document_search(self, search_terms: list, field: str) -> dict:
        # pudb.set_trace()
        d_resp: dict = {"acknowledged": False, "find_list": [], "error": ""}
        # pudb.set_trace()
        await self.create_text_index()

        regex = "|".join(
            search_terms
        )  # Create a regex pattern that matches any of the search terms
        async for doc in self.collection.find({}):
            for key, value in doc.items():
                if re.search(regex, str(value), re.IGNORECASE):
                    d_resp["find_list"].append(doc["_id"])
                    break

        if not len(d_resp["error"]):
            d_resp["acknowledged"] = True
        return d_resp

    async def collectionNames_list(self, cursor: AIO.AsyncIOMotorCursor) -> list:
        l_collection: list = []
        async for name in cursor:
            l_collection.append(name)
        return l_collection

    async def connectCollection(self, collection: str) -> dict:
        """
        Connect to the collection called <collection> (or create the
        collection if it does not exist). Return the collection and a
        bool exist_already/not_exist in a dictionary.

        :param mongocollection: the name of the collection
        :return: a dictionary with the collection, an element count, and
                    a status
        """
        error: str = ""
        connected: bool = True
        l_collection: list = []
        try:
            l_collection = await self.DB.list_collection_names()
        except Exception as e:
            # pudb.set_trace()
            error = f"{e}"
            connected = False
        d_ret: dict = {
            "connected": connected,
            "exists": True if collection in l_collection else False,
            "interface": self.DB[collection],
            "elements": 0,
            "error": error,
        }
        if d_ret["exists"]:
            filter: dict = {}
            d_ret["elements"] = await d_ret["interface"].count_documents(filter)
        return d_ret

    def __init__(self, DBobject: mongoDB) -> None:
        self.DB: AIO.AsyncIOMotorDatabase = DBobject.getDB()
        self.d_collection: dict = {}
        self.allowDuplicates: bool = DBobject.args.allowDuplicates
        self.addHashToDocument = not DBobject.args.noHashing
        self.donotFlatten = DBobject.args.donotFlatten

    def collection_serialize(self) -> responseModel.collectionDesc:
        # pudb.set_trace()
        resp: responseModel.collectionDesc = responseModel.collectionDesc()
        resp.databaseName = str(self.collection.database.name)
        resp.name = str(self.collection.name)
        resp.fullName = str(self.collection.full_name)
        resp.info.connected = self.d_collection["connected"]
        resp.info.existsAlready = self.d_collection["exists"]
        resp.info.elements = self.d_collection["elements"]
        resp.info.error = self.d_collection["error"]
        resp.info.status = resp.info.connected
        resp.status = resp.info.status
        return resp

    async def connect(self, name: str) -> responseModel.collectionDesc:
        self.d_collection: dict = await self.connectCollection(name)
        self.collection: AIO.AsyncIOMotorCollection = self.d_collection["interface"]
        return self.collection_serialize()



import os
from pydantic_settings import BaseSettings
from pfmongo.models import dataModel
from typing import Optional


class Mongo(BaseSettings):
    MD_URI: str = "localhost:27017"
    MD_DB: str = ""
    MD_COLLECTION: str = ""
    MD_username: str = "username"
    MD_password: str = "password"
    MD_sessionUser: str = ""
    stateDBname: str = "db.txt"
    stateColname: str = "collection.txt"
    stateDupname: str = "duplicates.txt"
    stateHashes: str = "hashes.txt"
    flattenSuffix: str = "-flat"
    responseTruncDepth: int = 4
    responseTruncSize: int = 6000
    responseTruncOver: int = 100000


class App(BaseSettings):
    logging: dataModel.loggingType = dataModel.loggingType.CONSOLE
    allowDuplicates: bool = False
    noHashing: bool = False
    donotFlatten: bool = False
    beQuiet: bool = False
    noResponseTruncSize: bool = False
    noComplain: bool = False
    detailedOutput: bool = False
    modelSizesPrint: bool = False
    eventLoopDebug: bool = False
    fontawesomeUse: bool = True

    class Config:
        env_prefix = "MD_"  # Matches environment variables that start with MD_
        case_sensitive = False  # Optional: Allows case-insensitive matching


logging_val: Optional[str] = "CONSOLE"
if "LOGGING" in os.environ:
    logging_val = os.environ["LOGGING"]
logging_enum: dataModel.loggingType = dataModel.loggingType.CONSOLE
if logging_val:
    try:
        logging_enum = dataModel.loggingType[logging_val]
    except KeyError:
        logging_enum = dataModel.loggingType.CONSOLE

mongosettings: Mongo = Mongo()
appsettings: App = App(logging=logging_enum)



import click
from argparse import Namespace
import pudb
from typing import Optional, Union, Annotated
from pfmisc import Colors as C
from pfmongo import env, pfmongo, driver
from pfmongo.models import responseModel
import asyncio

NC = C.NO_COLOUR
GR = C.LIGHT_GREEN
CY = C.CYAN


def stateResponse_eval(mongodb: pfmongo.Pfmongo) -> pfmongo.Pfmongo:
    mongodb.responseData = mongo_responseEmbed(state_get(mongodb.args))
    mongodb.exitCode = 0
    return mongodb


def state_get(options: Namespace) -> responseModel.stateResponse:
    state: responseModel.stateResponse = responseModel.stateResponse()
    state.database = env.DBname_get(options)
    state.collection = env.collectionName_get(options)
    return state


def mongo_responseEmbed(
    state: responseModel.stateResponse,
) -> responseModel.mongodbResponse:
    return pfmongo.responseData_build(
        {"status": True, "connect": state}, f"{state.database}/{state.collection}"
    )


async def showAll_asInt(options: Namespace) -> int:
    return await driver.run_intReturn(options, stateResponse_eval)


async def showAll_asModel(options: Namespace) -> responseModel.mongodbResponse:
    return await driver.run_modelReturn(options, stateResponse_eval)


def sync_showAll_asInt(options: Namespace) -> int:
    return asyncio.run(showAll_asInt(options))


def sync_showAll_asModel(options: Namespace) -> responseModel.mongodbResponse:
    return asyncio.run(showAll_asModel(options))


@click.command(
    cls=env.CustomCommand,
    help=f"""
show internal {GR}state{NC} values

SYNPOSIS
{CY}showall{NC}

DESC
This command shows internal program state. It accepts no arguments.

""",
)
@click.pass_context
def showAll(ctx: click.Context) -> int:
    # pudb.set_trace()
    return sync_showAll_asInt(ctx.obj["options"])



"""
Module for defining and registering the main Click application commands.

This module isolates the definition of the Click `app` and the registration
of all subcommands, providing a centralized and decoupled implementation
for the CLI application.
"""

import re
import click
from argparse import Namespace
from pfmongo import env
from pfmongo.pfmongo import options_initialize
from pfmongo.commands import database, collection, fs, state, document

# Global options namespace
options: Namespace = Namespace()


def namespace_pubattribs(space: Namespace) -> list[str]:
    """
    Retrieve public attributes from a Namespace object.

    :param space: The Namespace object to analyze.
    :return: A list of public attribute names.
    """
    attributes: list[str] = re.findall(r"\b([A-Za-z][A-Za-z_0-9]*)=", str(space))
    return attributes


@click.group(cls=env.CustomGroup, help="A client for interacting with MongoDB.")
@click.option("--man", is_flag=True, help="Show more detail about core OPTIONS.")
@click.option("--version", is_flag=True, help="Show program version.")
@click.pass_context
def app(ctx: click.Context, man: bool, version: bool) -> int:
    """
    Define the main Click application.

    This function sets up the CLI entry point using Click, allowing
    the registration of subcommands and providing base options like
    `--man` and `--version`. It also initializes and attaches the
    global `options` Namespace.

    :param ctx: The Click context object.
    :param man: Flag to show manual details.
    :param version: Flag to show version information.
    :return: 0 on success or an error code on failure.
    """
    global options

    # Initialize options if not already set
    if not namespace_pubattribs(options):
        options = options_initialize()

    # Attach options to the Click context
    ctx.obj = {}
    ctx.obj["options"] = options

    # Placeholder for tracking the invoked subcommand
    subcommand: str | None = click.get_current_context().invoked_subcommand

    return 0


# Register all commands
app.add_command(database.database)
app.add_command(collection.collection)
app.add_command(fs.fs)
app.add_command(state.state)
app.add_command(document.document)


# Import and register smash dynamically to avoid circular import
def register_smash() -> None:
    """
    Dynamically register the `smash` command to avoid circular imports.
    """
    from pfmongo.commands.smash import smash

    app.add_command(smash)


register_smash()



import asyncio
import click
from argparse import Namespace
from pfmisc import Colors as C
from pfmongo import driver, env
from pfmongo.models import responseModel
import copy
import pudb

NC = C.NO_COLOUR
GR = C.LIGHT_GREEN
CY = C.CYAN
PL = C.PURPLE


def options_add(options: Namespace) -> Namespace:
    localoptions: Namespace = copy.deepcopy(options)
    localoptions.do = "showAllDB"
    return localoptions


async def showAll_asInt(options: Namespace) -> int:
    return await driver.run_intReturn(options)


async def showAll_asModel(options: Namespace) -> responseModel.mongodbResponse:
    return await driver.run_modelReturn(options)


def sync_showAll_asInt(options: Namespace) -> int:
    return asyncio.run(showAll_asInt(options))


def sync_showAll_asModel(options: Namespace) -> responseModel.mongodbResponse:
    return asyncio.run(showAll_asModel(options))


@click.command(
    cls=env.CustomCommand,
    help=f"""
list {PL}DATABASES{NC} containing data

SYNOPSIS
{CY}showall{NC}

DESC
This command shows all the populated databases available in a given mongodb
server. It accepts no arguments.

""",
)
@click.pass_context
def showAll(ctx: click.Context) -> int:
    return sync_showAll_asInt(options_add(ctx.obj["options"]))



from    pfmisc      import  Colors  as C
from    pfmongo.commands.clop   import showAll, connect, deleteCol
from    pfmongo     import env
import  click
import  pudb

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

@click.group(cls=env.CustomGroup,  help = f"""
    commands suitable for collection focus: {GR}show, connect, deletecol{NC}

    This command group provides mongo {CY}"collection"{NC} level commands,
    primarily used to {CY}connect {NC}to a collection, {CY}showall{NC} collections,
    and also {CY}deletecol{NC} collections.

    Use the {CY}deletecol{NC} subcommand with care, since it will delete
    without asking for confirmation.
"""
)
def collection():
    pass

collection.add_command(showAll.showAll)
collection.add_command(connect.connect)
collection.add_command(deleteCol.deleteCol)



import click
import pudb
from pfmongo import driver, env
from argparse import Namespace
from pfmongo.models import responseModel
from pfmisc import Colors as C
import copy
import asyncio
from pfmongo.commands.clop import connect

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
PL = C.PURPLE
YL = C.YELLOW


async def options_add(options: Namespace) -> Namespace:
    localoptions: Namespace = copy.deepcopy(options)

    localoptions.do = "searchDocument"
    collectionName: Namespace = await connect.baseCollection_getAndConnect(options)
    localoptions.argument = {
        "field": "_id",
        "searchFor": options.pattern.split(","),
        "collection": collectionName.collectionName,
    }
    return localoptions


async def sg_asInt(options: Namespace) -> int:
    return await driver.run_intReturn(await options_add(options))


async def sg_asModel(options: Namespace) -> responseModel.mongodbResponse:
    return await driver.run_modelReturn(await options_add(options))


def sync_sg_asInt(options: Namespace) -> int:
    return asyncio.run(sg_asInt(options))


def sync_sg_asModel(options: Namespace) -> responseModel.mongodbResponse:
    return asyncio.run(sg_asModel(options))


@click.command(
    cls=env.CustomCommand,
    help=f"""
sipgrep {YL}pattern{NC}

SYNOPSIS
{CY}sg {YL}<pattern>{NC}

DESC
A pale shadow of "ripgrep", "sipgrep" aims to "sip", or perform an extremely
primitive and simple search across documents, returning the names of documents
(or files) that contain the search pattern.
""",
)
@click.pass_context
@click.argument("pattern", required=True)
def sg(ctx: click.Context, pattern: str) -> int:
    # pudb.set_trace()
    ctx.obj["options"].pattern = pattern
    return sync_sg_asInt(ctx.obj["options"])



from pfmongo import pfmongo
from pfmongo.__main__ import main
import os

from pfmongo.commands.dbop import connect as database
from pfmongo.commands.clop import connect as collection, showAll as colShow, deleteCol
from pfmongo.models import responseModel

os.environ["XDG_CONFIG_HOME"] = "/tmp"


def DB_connect(DB: str = "testDB") -> int:
    return database.sync_connectTo_asInt(
        database.options_add(DB, pfmongo.options_initialize())
    )


def test_collection_connect_moduleAsInt() -> None:
    """connect to a collection "testCollection" using a module call with an int return"""
    DB_connect()
    ret: int = collection.sync_connectTo_asInt(
        collection.options_add("testCollection", pfmongo.options_initialize())
    )
    assert ret == 0


def test_collection_connect_main(capsys) -> None:
    """connect to a collection via click"""
    ret: int = 0
    DB_connect()
    try:
        ret = main("collection connect testCollection".split())
    except SystemExit:
        pass
    captured = capsys.readouterr()
    assert ret == 0
    assert 'Successfully connected collection to "testCollection"' in captured.out


def test_collection_connect_moduleAsModel() -> None:
    """connect to a collection "testCollection" using a module call with model return"""

    DB_connect()
    ret: responseModel.mongodbResponse = collection.sync_connectTo_asModel(
        collection.options_add("testCollection", pfmongo.options_initialize())
    )
    assert "Successfully" in ret.message


def test_collection_showall_main(capsys) -> None:
    """show all collections via click"""
    ret: int = 0
    DB_connect("admin")
    try:
        ret = main("--useDB admin collection showall".split())
    except SystemExit:
        pass
    captured = capsys.readouterr()
    assert ret == 0
    assert "system" in captured.out


def test_collection_showall_moduleAsInt() -> None:
    """show all collections using a module call with an int return"""

    DB_connect("admin")
    ret: int = colShow.sync_showAll_asInt(
        colShow.options_add(pfmongo.options_initialize())
    )
    assert ret == 0


def test_collection_showAll_moduleAsModel() -> None:
    """show all collections using a module call with model return"""
    DB_connect("admin")
    ret: responseModel.mongodbResponse = colShow.sync_showAll_asModel(
        colShow.options_add(pfmongo.options_initialize())
    )
    assert "system" in ret.message


def test_collection_delete_main(capsys) -> None:
    """delete collection via click"""
    ret: int = 0
    try:
        ret = main(
            "--useCollection testCollection collection deletecol testCollection".split()
        )
    except SystemExit:
        pass
    captured = capsys.readouterr()
    assert ret == 0
    assert "Successfully" in captured.out


def test_collection_delete_moduleAsInt() -> None:
    """delete databases using a module call with an int return"""

    ret: int = deleteCol.sync_collectiondel_asInt(
        deleteCol.options_add("testCollection", pfmongo.options_initialize())
    )
    assert ret == 0


def test_collection_delete_moduleAsModel() -> None:
    """show all databases using a module call with model return"""

    ret: responseModel.mongodbResponse = deleteCol.sync_collectiondel_asModel(
        deleteCol.options_add("testCollection", pfmongo.options_initialize())
    )

    assert "Successfully" in ret.message



from re import A
import sys
import socket
from argparse import Namespace, ArgumentParser, RawTextHelpFormatter
from pfmongo import pfmongo
from pfmongo.app_core import app
from pfmongo.pfmongo import options_initialize
from pfmongo.commands import smash
import pudb
from typing import Any
import asyncio
import aiohttp
from pfmisc import Colors as C
from copy import deepcopy
from pathlib import Path
from pydantic import BaseModel
import asyncio
from urllib.parse import quote
import re

LG = C.LIGHT_GREEN
YL = C.YELLOW
NC = C.NO_COLOUR

TERMINATION_SEQUENCE = b"\r\n\r\n"


def parser_setup(str_desc: str = "") -> ArgumentParser:
    description: str = ""
    if len(str_desc):
        description = str_desc
    parser = ArgumentParser(
        description=description, formatter_class=RawTextHelpFormatter
    )

    parser.add_argument(
        "--host",
        type=str,
        default="",
        help="host name or IP of server",
    )

    parser.add_argument(
        "--port",
        type=str,
        default="",
        help="port on which remote server is listening",
    )

    parser.add_argument(
        "--server",
        default=False,
        action="store_true",
        help="If specified, run in server mode",
    )

    parser.add_argument(
        "--response",
        type=str,
        default="string",
        help="response type: either a 'string' or 'dict'",
    )

    parser.add_argument(
        "--url",
        type=str,
        default="http://localhost:8025/api/v1/",
        help="address of a pfmdb server",
    )

    parser.add_argument(
        "--msg", type=str, default="", help="message to transmit in client mode"
    )

    parser.add_argument(
        "--repl",
        default=False,
        action="store_true",
        help="If specified, run the client in REPL mode",
    )
    return parser


def parser_interpret(parser: ArgumentParser, *args, **kwargs) -> Namespace:
    """
    Interpret the list space of *args, or sys.argv[1:] if
    *args is empty
    """
    options: Namespace = Namespace()
    asModule: bool = False
    for k, v in kwargs.items():
        if k == "asModule":
            asModule = v
    if asModule:
        # Here, this code is used a module to another app
        # and we don't want to "interpret" the host app's
        # CLI.
        options, unknown = parser.parse_known_args()
        return options
    if len(args):
        if len(args[0]):
            if isinstance(args[0][0], list):
                options = parser.parse_args(args[0][0])
            elif isinstance(args[0][0], dict):
                options = parser.parse_args(parser_JSONinterpret(args[0][0]))
        else:
            options = parser.parse_args(sys.argv[1:])
    return options


def parser_JSONinterpret(d_JSONargs) -> list:
    """
    Interpret a JSON dictionary in lieu of CLI.

    For each <key>:<value> in the d_JSONargs, append to
    list two strings ["--<key>", "<value>"] and then
    argparse.
    """
    l_args = []
    for k, v in d_JSONargs.items():
        if isinstance(v, bool):
            if v:
                l_args.append("--%s" % k)
            continue
        l_args.append("--%s" % k)
        l_args.append("%s" % v)
    return l_args


def nixToWin_path(cmd: str) -> str:
    """
    Any cmd from the client that contains a windows style '\\'
    has this replaced with '/'
    """
    return re.sub(r"/(?!/)", r"\\", cmd)


def path_to_dbCol(path: Path) -> tuple:
    db: str = ""
    collection: str = ""
    match len(path.parts):
        case _ if len(path.parts) == 2:
            (root, db) = path.parts
        case _ if len(path.parts) > 2:
            (root, db, collection) = path.parts[:3]
    return (db, collection)


class FastAPIparams(BaseModel):
    database: str
    collection: str
    handler: str = "smash"


class FastAPIheader(BaseModel):
    headers: str = "accept: application/json"


class FastAPIpayload(BaseModel):
    url: str
    params: FastAPIparams
    headers: FastAPIheader
    data: str = ""

    def msg_parse(self, msg: str) -> tuple[str, str, str]:
        args: list[str] = msg.split()
        path_index: int
        database: str = ""
        collection: str = ""

        if msg.startswith("cd"):
            msg = nixToWin_path(msg)
            return quote(msg), database, collection

        # Find the index of the last argument that starts with '/'
        for i in range(len(args) - 1, -1, -1):
            if args[i].startswith("/"):
                path_index = i
                break
        else:
            # No path found, return the original command
            return quote(msg), database, collection
        last_arg = args.pop(path_index)
        (database, collection) = path_to_dbCol(Path(last_arg))
        path_prefix, file_name = last_arg.rsplit("/", 1)
        args.append(file_name)
        msg = " ".join(args)
        return quote(msg), database, collection

    def __init__(self, url: str, msg: str, **data):
        cmd: str
        database: str
        collection: str
        cmdurl: str
        cmd, database, collection = self.msg_parse(msg)
        cmdurl = f"{url}/{cmd}"
        super().__init__(
            url=cmdurl,
            params=FastAPIparams(database=database, collection=collection),
            headers=FastAPIheader(),
        )


class FastAPIresponse(BaseModel):
    status: int
    response: Any


class FastAPIclient:
    def __init__(self, url: str):
        self.url = f"{url}pfmongo/cli"

    async def message_sendAndReceive(self, msg: str) -> FastAPIresponse:
        pudb.set_trace()
        fastAPIpayload: FastAPIpayload = FastAPIpayload(self.url, msg)
        status: int
        payload: Any

        session: aiohttp.ClientSession = aiohttp.ClientSession()
        try:
            async with session.post(
                fastAPIpayload.url,
                params=fastAPIpayload.params.model_dump(),
                headers=fastAPIpayload.headers.model_dump(),
                data=fastAPIpayload.data,
            ) as response:
                status = response.status
                if response.headers.get("Content-Type", "").startswith(
                    "application/json"
                ):
                    payload = await response.json()
                else:
                    payload = await response.text()

        finally:
            await session.close()

        return FastAPIresponse(status=status, response=payload)


class IPCclient:
    def __init__(self, host: str, port: str):
        self.clientSocket: socket.socket = socket.socket(
            socket.AF_INET, socket.SOCK_STREAM
        )
        self.clientSocket.connect((host, int(port)))

    async def message_sendAndReceive(self, msg: str) -> dict[str, str]:
        resp: dict[str, str] = {"response": ""}
        result: str = ""
        try:
            self.clientSocket.sendall(msg.encode())
            response: bytes = b""
            while True:
                chunk: bytes = self.clientSocket.recv(1024)
                if not chunk:
                    break
                response += chunk
            if response:
                result = response.decode()
            else:
                result = "No response received"
            resp["response"] = result
        finally:
            self.clientSocket.close()

        return resp


class IPCserver:
    def __init__(self, host: str, port: str):
        self.serverSocket: socket.socket = socket.socket(
            socket.AF_INET, socket.SOCK_STREAM
        )
        self.serverSocket.bind((host, int(port)))
        self.serverSocket.listen(1)
        self.connection: socket.socket
        self.clientAddress: tuple[str, str]
        print(f"smashes server setup and listening on '{host}:{port}'")

    async def response_process(self, incoming: str) -> str:
        response: str | bytes
        ret: str = ""
        mdbOptions: Namespace = options_initialize()
        response = await smash.smash_execute_async(
            smash.command_parse(
                await smash.command_get(mdbOptions, noninteractive=incoming)
            ),
            smash.pipe_handler,
        )
        if isinstance(response, str):
            ret = response
        if isinstance(response, bytes):
            ret = response.decode()
        print(f"{ret}")
        return ret

    def response_await(self) -> str:
        incoming: str = ""
        resp: str = ""
        self.connection, self.clientAddress = self.serverSocket.accept()
        try:
            print(f"Connection from {self.clientAddress}")
            data: bytes = b""
            data = self.connection.recv(32768)
            if data:
                incoming = data.decode()
                print(f"Received: {incoming}")
                resp = asyncio.run(self.response_process(incoming))
                self.connection.sendall(resp.encode())
        finally:
            self.connection.close()

        return resp

    def start(self) -> None:
        while True:
            incoming: str = self.response_await()


def server_handle(options: Namespace) -> None:
    server: IPCserver = IPCserver(options.host, options.port)
    server.start()


def options_msg(options: Namespace, msg: str) -> Namespace:
    localOptions: Namespace = deepcopy(options)
    localOptions.msg = msg
    return localOptions


async def client_handle(options: Namespace) -> dict[str, str]:
    client: IPCclient | FastAPIclient
    if options.url:
        client = FastAPIclient(options.url)
    else:
        client = IPCclient(options.host, options.port)
    pudb.set_trace()
    return await client.message_sendAndReceive(options.msg)


def response_toConsole(resp: dict[str, str]) -> str:
    return resp["response"]


def prompt_prefix(options: Namespace) -> str:
    if options.host:
        return f"{YL}[{options.host}:{options.port}]{NC}"
    else:
        return f"{YL}[options:url]{NC}"


async def client_repl(options: Namespace) -> None:
    pfOptions: Namespace = options_initialize()
    pudb.set_trace()
    while True:
        remoteLS: dict[str, str] = await client_handle(options_msg(options, "ls --raw"))
        print(f"{prompt_prefix(options)}")
        command: str = await smash.command_get(pfOptions, files=remoteLS["response"])
        if "exit" in command.lower():
            break
        dresp: dict[str, str] = await client_handle(options_msg(options, command))
        resp: str = response_toConsole(dresp)
        if "No response received" not in resp:
            print(resp)


def main(*args: list[Any]) -> str | dict[str, str]:
    options: Namespace = parser_interpret(parser_setup(), args)

    dresp: dict[str, str]
    if options.server:
        server_handle(options)

    if options.repl:
        asyncio.run(client_repl(options))
        return "0"

    dresp = asyncio.run(client_handle(options))
    if "string" in options.response:
        return response_toConsole(dresp)
    return dresp


if __name__ == "__main__":
    main()



from typing import Callable, Optional, Coroutine
from argparse import Namespace
import asyncio
from asyncio import AbstractEventLoop
import json
import sys
from pfmisc import Colors as C
from pfmongo import pfmongo
from pfmongo.models import responseModel, dataModel
from pfmongo.pfmongo import Pfmongo as MONGO
from pfmongo.config import settings
import copy
import pudb

from typing import Any, Dict, List, Union, cast
from pydantic import BaseModel

try:
    from . import __pkg, __version__
except:
    from pfmongo import __pkg, __version__

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN

# Define a new type that includes all possibilities
NestedDict = Union[str, Dict[str, Any], List[Any]]


class SizeLimitedDict(BaseModel):
    value: NestedDict


def settmp(options: Namespace, newkeyvaluepair: list) -> Namespace:
    """
    set some values (newkeyvaluepair) in a copy of
    <options> and return this copy. Note the original
    <options> is not affected.
    """
    localoptions = copy.deepcopy(options)
    for pair in newkeyvaluepair:
        for k, v in pair.items():
            setattr(localoptions, k, v)
    return localoptions


def get_size(obj: NestedDict) -> int:
    size = sys.getsizeof(obj)
    if isinstance(obj, dict):
        size += sum([get_size(v) for v in obj.values()])
        size += sum([get_size(k) for k in obj.keys()])
    elif hasattr(obj, "__dict__"):
        size += get_size(obj.__dict__)
    elif hasattr(obj, "__iter__") and not isinstance(obj, (str, bytes, bytearray)):
        size += sum([get_size(i) for i in obj])
    return size


def size_limit(obj: Any, limit: int, depth: int) -> NestedDict:
    # if depth == 0 and sys.getsizeof(obj) > limit:
    size: int = get_size(obj)
    if depth == 0 and size > limit:
        # return "size too large"
        return f">>>truncated<<<({str(size)} > {limit})"
    elif isinstance(obj, dict):
        return {k: size_limit(v, limit, depth - 1) for k, v in obj.items()}
    elif isinstance(obj, list):
        return [size_limit(elem, limit, depth - 1) for elem in obj]
    else:
        return obj


def model_pruneForDisplay(model: NestedDict) -> NestedDict:
    if not settings.appsettings.noResponseTruncSize:
        model = size_limit(
            model,
            settings.mongosettings.responseTruncSize,
            settings.mongosettings.responseTruncDepth,
        )
    depthUp: int = 1
    while get_size(model) > settings.mongosettings.responseTruncOver:
        model = size_limit(
            model,
            settings.mongosettings.responseTruncSize,
            settings.mongosettings.responseTruncDepth - depthUp,
        )
        depthUp += 1
    return model


def model_gets(mongodb: MONGO) -> Callable[[bool], str]:
    """return the internal response model as a string"""
    model: NestedDict = {}
    modelForDisplay: NestedDict = {}

    def model_toStr(addModelSizes: bool = False) -> str:
        respstr: str = ""
        if not settings.appsettings.detailedOutput:
            respstr = mongodb.responseData.message
            if settings.appsettings.logging == dataModel.loggingType.NDJSON:
                respstr = f'{{"pfmongo": "{respstr}"}}'
            return respstr
        try:
            respstr = mongodb.responseData.model_dump_json()
        except Exception as e:
            respstr = "%s" % mongodb.responseData.model_dump()
        model = json.loads(respstr)
        modelForDisplay = model_pruneForDisplay(model)
        respstr = json.dumps(modelForDisplay)
        if settings.appsettings.logging == dataModel.loggingType.NDJSON:
            respstr = f'{{"pfmongo": {respstr}}}'
        if addModelSizes:
            respstr += json.dumps(
                {
                    "modelSize": {
                        "orig": get_size(model),
                        "disp": get_size(modelForDisplay),
                    }
                }
            )
        return respstr

    return model_toStr


def responseData_print(mongodb: MONGO) -> None:
    model_asString: Callable[[bool], str] = model_gets(mongodb)
    print(model_asString(settings.appsettings.modelSizesPrint))


def event_setup(
    options: Namespace,
    f_syncCallBack: Optional[Callable[[MONGO], MONGO]] = None,
    # ) -> Callable[..., int | responseModel.mongodbResponse]:
) -> Callable[..., Coroutine[None, None, int | responseModel.mongodbResponse]]:
    # Create the mongodb object...
    mongodb: pfmongo.Pfmongo = pfmongo.Pfmongo(options)

    def payloadAs(returnType: str = "int") -> int | responseModel.mongodbResponse:
        match returnType:
            case "int":
                return mongodb.exitCode
            case "model":
                mongodb.responseData.exitCode = mongodb.exitCode
                return mongodb.responseData
            case _:
                return mongodb.exitCode

    async def run(**kwargs) -> int | responseModel.mongodbResponse:
        nonlocal mongodb
        printResponse: bool = False
        returnType: str = "int"
        for k, v in kwargs.items():
            if k == "printResponse":
                printResponse = v
            if k == "returnType":
                returnType = v

        if not f_syncCallBack:
            # run it asynchronously..!
            loop: AbstractEventLoop

            # If an event loop already exists, use it!
            try:
                loop = asyncio.get_event_loop()
            except RuntimeError:
                # Else create a new loop
                loop = asyncio.new_event_loop()
                asyncio.set_event_loop(loop)
            task = loop.create_task(mongodb.service())
            await task
            # try:
            #     loop.run_until_complete(mongodb.service())
            # except Exception as e:
            #     print(f"In event_setup/run: {e}")
        else:
            # else run it with a synchronous callback
            mongodb = f_syncCallBack(mongodb)
        if printResponse and not options.beQuiet:
            responseData_print(mongodb)
        return payloadAs(returnType)

    return run


async def do(
    options: Namespace,
    retType: str,
    f_syncCallBack: Optional[Callable[[MONGO], MONGO]] = None,
) -> int | responseModel.mongodbResponse:
    f = event_setup(options, f_syncCallBack)
    return await f(printResponse=True, returnType=retType)


async def run_intReturn(
    options: Namespace, f_syncCallBack: Optional[Callable[[MONGO], MONGO]] = None
) -> int:
    if not isinstance((result := await do(options, "int", f_syncCallBack)), int):
        raise TypeError("did not receive int as expected")
    return result


async def run_modelReturn(
    options: Namespace, f_syncCallBack: Optional[Callable[[MONGO], MONGO]] = None
) -> responseModel.mongodbResponse:
    if not isinstance(
        (result := await do(options, "model", f_syncCallBack)),
        responseModel.mongodbResponse,
    ):
        raise TypeError("did not receive model as expected")
    return result


def run(
    options: Namespace, f_syncCallBack: Optional[Callable[[MONGO], MONGO]] = None
) -> int:
    # Create the mongodb object...
    mongodb: pfmongo.Pfmongo = pfmongo.Pfmongo(options)

    if not f_syncCallBack:
        # run it asynchronously..!
        loop: AbstractEventLoop = asyncio.get_event_loop()
        loop.run_until_complete(mongodb.service())
    else:
        # else run it with a synchronous callback
        mongodb = f_syncCallBack(mongodb)

    # print responses...
    responseData_print(mongodb)

    # and we're done.
    return mongodb.exitCode



import click
import pudb
from pfmongo import driver, env
from argparse import Namespace
from pfmisc import Colors as C
from pfmongo.models import responseModel
from typing import Required, cast
import copy
from pfmongo.commands.clop import connect
import asyncio

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
YL = C.YELLOW
PL = C.PURPLE


def options_add(target: str, field: str, options: Namespace) -> Namespace:
    localoptions: Namespace = copy.deepcopy(options)
    localoptions.do = "searchDocument"
    localoptions.argument = {
        "field": field,
        "searchFor": target.split(","),
        "collection": asyncio.run(
            connect.baseCollection_getAndConnect(options)
        ).collectionName,
    }
    return localoptions


async def documentSearch_asInt(options: Namespace) -> int:
    return await driver.run_intReturn(options)


async def documentSearch_asModel(options: Namespace) -> responseModel.mongodbResponse:
    return await driver.run_modelReturn(options)


def sync_documentSearch_asInt(options: Namespace) -> int:
    return asyncio.run(documentSearch_asInt(options))


def sync_documentSearch_asModel(options: Namespace) -> responseModel.mongodbResponse:
    return asyncio.run(documentSearch_asModel(options))


@click.command(
    cls=env.CustomCommand,
    help=f"""
find all {PL}documents{NC} containing strings of interest

SYNOPSIS
{CY}search {GR}--target {YL}<stringList>{NC} [{GR}--field {YL}<field>{NC}]

DESC
Search across all documents in a {YL}collection{NC} for any that contain
substrings in the {YL}<stringList>{NC}. This is a comma separated string
of targets to find.

If a {YL}document{NC} contains one (or more) of the {YL}<stringList>{NC} targets
return the value of the {YL}document{NC}'s {YL}<field>{NC}. Typically this is the
document {YL}_id{NC}.

""",
)
@click.option(
    "--target",
    type=str,
    help="A comma separated list. The logical OR of the search is returned",
    default="",
)
@click.option(
    "--field",
    type=str,
    help="List the search hits referenced by this field",
    default="_id",
    required=False,
)
@click.pass_context
def search(ctx: click.Context, target: str, field: str) -> int:
    # pudb.set_trace()
    return documentSearch_asInt(options_add(target, field, ctx.obj["options"]))



import click
import pudb
from typing import Any, Optional, Callable, Union
import sys
from tabulate import tabulate
from argparse import Namespace
from pfmongo import driver, env
from pfmongo.commands.state import showAll as state
from pfmisc import Colors as C
from pfmongo.models import responseModel
from click.testing import CliRunner
from asyncio import get_event_loop
from pathlib import Path
from fortune import fortune
from pfmongo.config import settings
import subprocess
import asyncio
from pydantic import BaseModel
from pfmongo.commands.slib import tabc
import pfmongo.commands.fop.prompt as prompt

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
YL = C.YELLOW

fscommand: list = [
    "sg",
    "ls",
    "cat",
    "rm",
    "cd",
    "mkcd",
    "imp",
    "exp",
    "prompt",
    "pwd",
    "quit",
    "exit",
    "fortune",
    "banner",
]

fscommand_noArgs: list = ["prompt", "pwd", "quit", "exit"]


def get_app():
    """
    Retrieve the Click application instance.

    This prevents circular imports by delaying the import of `app` until runtime.

    :return: The Click application instance.
    """
    from pfmongo.app_core import app

    return app


def pipe_split(command: str) -> list[str]:
    """
    Split a command string by the pipe (|) character.

    :param command: The command string to split.
    :return: A list containing the split components.
    """
    parts: list[str] = command.split("|", 1)
    return parts


def smash_output(command: str) -> str:
    """
    Execute a command and capture its output.

    :param command: The command string to execute.
    :return: The output of the command execution.
    """
    output: str = meta_parse(command)
    if not output:
        ret = CliRunner().invoke(get_app(), command.split(), color=True)
        output = ret.output
    return output


class ClickResult(BaseModel):
    exit_code: int
    output: str
    # Add other fields if needed


async def invoke_click_async(app, args: list[str], **kwargs) -> ClickResult:
    """
    Asynchronously invoke a Click command.

    :param app: The Click application.
    :param args: List of command arguments.
    :param kwargs: Additional keyword arguments for CliRunner.invoke.
    :return: A ClickResult instance with the command's result.
    """
    loop = asyncio.get_running_loop()
    runner = CliRunner()
    result = await loop.run_in_executor(
        None, lambda: runner.invoke(app, args, **kwargs)
    )
    return ClickResult(exit_code=result.exit_code, output=result.output)


async def smash_output_async(command: str) -> str:
    """
    Asynchronous version of smash_output.

    :param command: The command string to process.
    :return: The output of the command.
    """
    output: str = meta_parse(command)
    if not output:
        result: ClickResult = await invoke_click_async(
            get_app(), command.split(), color=True
        )
        output = result.output
    return output


async def smash_execute_async(
    command: str,
    f: Optional[Callable[[str, list[str]], subprocess.CompletedProcess]] = None,
) -> Union[bytes, str]:
    """
    Asynchronous version of smash_execute.

    :param command: The command string to process.
    :param f: Optional callable to handle piped commands.
    :return: Result of the command execution.
    """
    cmdpart: list[str] = pipe_split(command)
    smash_ret: str = await smash_output_async(cmdpart[0])
    result: str = smash_ret

    if len(cmdpart) > 1 and f:
        # Handle potential blocking operation in a separate thread
        loop = asyncio.get_running_loop()
        process: subprocess.CompletedProcess = await loop.run_in_executor(
            None, lambda: f(smash_ret, cmdpart)
        )
        result = f"exec: '{process.args}', returncode: {process.returncode}"
        result = ""

    return result


def smash_execute(
    command: str,
    f: Optional[Callable[[str, list[str]], subprocess.CompletedProcess]] = None,
) -> Union[bytes, str]:
    """
    Execute a command synchronously.

    :param command: The command string to execute.
    :param f: Optional callable to handle piped commands.
    :return: The result of the command execution.
    """
    cmdpart: list[str] = pipe_split(command)
    smash_ret: str = smash_output(cmdpart[0])
    result: str = smash_ret
    if len(cmdpart) > 1 and f:
        process: subprocess.CompletedProcess = f(smash_ret, cmdpart)
        result = f"exec: '{process.args}', returncode: {process.returncode}"
        result = ""
    return result


async def smash_executeAsync(
    command: str,
    f: Optional[Callable[[str, list[str]], subprocess.CompletedProcess]] = None,
) -> Union[bytes, str]:
    """
    Another asynchronous execution method.

    :param command: The command string to process.
    :param f: Optional callable to handle piped commands.
    :return: The result of the command execution.
    """
    cmdpart: list[str] = pipe_split(command)
    smash_ret: str = smash_output(cmdpart[0])
    result: str = smash_ret
    if len(cmdpart) > 1 and f:
        process: subprocess.CompletedProcess = f(smash_ret, cmdpart)
        result = f"exec: '{process.args}', returncode: {process.returncode}"
        result = ""
    return result


def pipe_handler(
    previous_input: str, cmdpart: list[str]
) -> subprocess.CompletedProcess:
    """
    Handle piped shell commands.

    :param previous_input: Input string from the previous command.
    :param cmdpart: List of command parts.
    :return: A completed subprocess process.
    """
    cmds = [c.strip() for c in cmdpart]
    shell_command = "|".join(cmds[1:])
    return subprocess.run(
        shell_command, input=previous_input, shell=True, capture_output=False, text=True
    )


def command_parse(command: str) -> str:
    """
    Parse a command string and prepend "fs" if it matches known commands.

    :param command: The command string.
    :return: The parsed command string.
    """
    fscall = [s for s in fscommand if command.lower().startswith(s)]
    if fscall:
        command = f"fs {command}"
    if command == "help":
        command = "fs --help"
    return command


async def state_getModel(options: Namespace) -> responseModel.mongodbResponse:
    """
    Fetch the current MongoDB state model asynchronously.

    :param options: Namespace containing CLI options.
    :return: The MongoDB state model.
    """
    return await state.showAll_asModel(driver.settmp(options, [{"beQuiet": True}]))


async def cwd(options: Namespace) -> Path:
    """
    Get the current working directory based on MongoDB state.

    :param options: Namespace containing CLI options.
    :return: The current working directory as a Path object.
    """
    model = await state_getModel(options)
    if model.message == "/":
        return Path("/")
    return Path("/" + model.message)


async def prompt_get(options: Namespace, context: str = "smash") -> str:
    """
    Generate a shell prompt string based on the current state.

    :param options: Namespace containing CLI options.
    :param context: Optional string representing the shell context (default: "smash").
    :return: The formatted shell prompt string.
    """
    promptDo = await prompt.prompt_do(await prompt.options_add(options))
    pathColor = promptDo.message
    return f"{CY}({settings.mongosettings.MD_sessionUser}@{context}){NC}{pathColor}$>"


async def command_get(options: Namespace, **kwargs) -> str:
    """
    Asynchronously fetch user input for commands.

    :param options: Namespace containing CLI options.
    :param kwargs: Additional keyword arguments for input handling.
    :return: The user input as a string.
    """
    userInput = await tabc.userInput_get(options, **kwargs)
    return userInput.strip()


def sync_command_get(options: Namespace, **kwargs) -> str:
    """
    Synchronously fetch user input for commands.

    :param options: Namespace containing CLI options.
    :param kwargs: Additional keyword arguments for input handling.
    :return: The user input as a string.
    """
    return asyncio.run(command_get(options, **kwargs))


def meta_parse(command: str) -> str:
    """
    Parse meta commands like "quit", "banner", or "fortune".

    :param command: The command string.
    :return: Parsed output if the command matches; otherwise, an empty string.
    """
    if "quit" in command.lower() or "exit" in command.lower():
        sys.exit(0)
    if "banner" in command.lower():
        return introbanner_generate()
    if "fortune" in command.lower():
        return env.tabulate_message(fortune(), f"{YL}fortune{NC}")
    return ""


def introbanner_generate() -> str:
    """
    Generate the introductory banner for the shell.

    :return: The formatted banner string.
    """
    title = f"{CY}s{YL}imple pf{CY}m{YL}ongo{NC} {CY}a{YL}pplication {CY}sh{YL}ell{NC}"
    banner = f"""
{CY}░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░{NC}
{CY}░███████╗███╗░░░███╗░█████╗░███████╗██╗░░██╗██╗░{NC}
{CY}░██╔════╝████╗░████║██╔══██╗██╔════╝██║░░██║██║░{NC}
{CY}░███████╗██╔████╔██║███████║███████╗███████║██║░{NC}
{CY}░╚════██║██║╚██╔╝██║██╔══██║╚════██║██╔══██║╚═╝░{NC}
{CY}░███████║██║░╚═╝░██║██║░░██║███████║██║░░██║██╗░{NC}
{CY}░╚══════╝╚═╝░░░░░╚═╝╚═╝░░╚═╝╚══════╝╚═╝░░╚═╝╚═╝░{NC}
{CY}░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░{NC}
"""
    intro = f"""
Welcome to {CY}smash{NC}, a simple shell to {YL}pfmongo{NC} that
allows you to directly call various subcommands.

If you really want to {CY}smash{NC} things up, the shell allows
you to call the {YL}fs{NC} subcommands without the {YL}fs{NC}, giving
the impression of being in a more standard un*x land.

Some useful commands:
 ▙▄ {YL}--help{NC} for a list of {YL}all{NC} commands.
 ▙▄ {YL}banner{NC} to see this amazing banner again.
 ▙▄ {YL}fortune{NC} since every shell needs this.
 ▙▄ {YL}help{NC} for a list of {YL}fs{NC} commands.
 ▙▄ {YL}quit{NC} or {YL}exit{NC} to return to the system.

Enjoy your stay and please remain on the trails!
Oh, and don't feed the wildlife. It's not good
for them.

Have a {CY}smash{NC}ing good time!
"""
    return env.tabulate_message(banner + intro, title)


@click.command(
    cls=env.CustomCommand,
    help="""
shell interface for running commands

An extremely "simple" pfmongo {YL}shell{NC}. Run commands from a shell-esque
interface that harkens back to the days of /bin/ash!

""",
)
@click.option("--prompt", is_flag=True, help="If set, print the CFS cwd as prompt")
@click.pass_context
def smash(ctx: click.Context, prompt) -> None:
    """
    Launch the shell interface for running commands.

    :param ctx: Click context.
    :param prompt: Flag to display the CFS prompt.
    """
    print(introbanner_generate())
    options = ctx.obj["options"]
    loop = get_event_loop()
    while True:
        command_str = loop.run_until_complete(command_get(options))
        print(smash_execute(command_parse(command_str), pipe_handler))



"""
Module for handling the `--man` and `--version` CLI options.

This module provides helper functions to display the manual page and version
information for the pfmongo CLI application.
"""

from argparse import Namespace
from pfmongo import driver, pfmongo
from pfmongo.pfmongo import package_CLIfull, package_argsSynopsisFull
from pfmisc import Colors as C
import os
import sys

NC = C.NO_COLOUR

str_title: str = r"""

           $$$$$$\
          $$  __$$\
 $$$$$$\  $$ /  \__|$$$$$$\$$$$\   $$$$$$\  $$$$$$$\   $$$$$$\   $$$$$$\
$$  __$$\ $$$$\     $$  _$$  _$$\ $$  __$$\ $$  __$$\ $$  __$$\ $$  __$$\
$$ /  $$ |$$  _|    $$ / $$ / $$ |$$ /  $$ |$$ |  $$ |$$ /  $$ |$$ /  $$ |
$$ |  $$ |$$ |      $$ | $$ | $$ |$$ |  $$ |$$ |  $$ |$$ |  $$ |$$ |  $$ |
$$$$$$$  |$$ |      $$ | $$ | $$ |\$$$$$$  |$$ |  $$ |\$$$$$$$ |\$$$$$$  |
$$  ____/ \__|      \__| \__| \__| \______/ \__|  \__| \____$$ | \______/
$$ |                                                  $$\   $$ |
$$ |                                                  \$$$$$$  |
\__|                                                   \______/
"""


str_heading: str = f"""
                        python (pf) mongodb client and module

"""


def synopsis_print(summary: bool = False) -> int:
    """
    Print the synopsis of the application, including core usage.

    :param summary: If True, only print a summary.
    :return: Exit code (1 for summary, 2 for full output).
    """
    scriptName: str = os.path.basename(sys.argv[0])
    print(C.CYAN + "\nNAME\n" + NC, end="")
    print(scriptName)
    print(C.CYAN + "\nCORE SYNOPSIS\n" + NC, end="")
    print(scriptName + pfmongo.package_CLIfull)
    print(C.CYAN + "\nDESCRIPTION\n" + NC, end="")
    print(pfmongo.package_coreDescription)

    if summary:
        return 1
    print(C.CYAN + "\nARGS\n" + NC, end="")
    print(pfmongo.package_argsSynopsisFull)
    return 2


def coreOptions_show(options: Namespace) -> bool:
    """
    Check if the `--man` or `--version` options are set.

    :param options: The parsed CLI options.
    :return: True if `--man` or `--version` is set, otherwise False.
    """
    return options.man or options.version


def versionAndName_print() -> int:
    """
    Print the package name and version.

    :return: Exit code (1 for successful version display).
    """
    from pfmongo import __pkg_name__, __version__  # Use updated attributes

    print("Name:    ", end="")
    print(C.LIGHT_GREEN + f"{__pkg_name__}" + NC)
    print("Version: ", end="")
    print(C.LIGHT_CYAN + f"{__version__}" + NC)
    return 1


def coreManpage_print() -> int:
    """
    Print the core manual page.

    :return: Exit code (2 for successful manual display).
    """
    print(f"{C.GREEN}{str_title}{str_heading}{NC}")
    return synopsis_print(summary=False)


def man(options: Namespace) -> int:
    """
    Handle the `--man` and `--version` options.

    :param options: The parsed CLI options.
    :return: Exit code depending on the option:
             1 for `--version`, 2 for `--man`, 0 otherwise.
    """
    if options.version:
        return versionAndName_print()
    if options.man:
        return coreManpage_print()
    return 0



"""
Test suite for the pfmongo CLI.

This module tests core functionalities of the pfmongo CLI, including
commands like `--man`, `--version`, and specific subcommand help options.
It uses pytest and the `capsys` fixture to capture and validate output.

Helper functions:
- CLIcall_parseForStringAndRet: Simplifies the testing of CLI commands by
  running the command, capturing its output, and verifying expected results.

Tests:
- test_main_manCore: Validates the `--man` option for core manual page output.
- test_main_version: Verifies the `--version` option outputs version info.
- test_imp_help: Ensures the `fs imp --help` command outputs the expected help text.
"""

import builtins
from _pytest.capture import capsys
import pytest
from pfmongo.__main__ import cli_entry_point
import os

# Set the XDG_CONFIG_HOME for temporary testing purposes
os.environ["XDG_CONFIG_HOME"] = "/tmp"


def CLIcall_parseForStringAndRet(
    capsys, cli: str, contains: str, exitCode: int
) -> None:
    """
    Run a CLI command, capture its output, and validate results.

    This helper function invokes the CLI command, checks its exit code, and
    verifies that the output contains the expected substring.

    :param capsys: pytest capture fixture for capturing stdout/stderr.
    :param cli: The CLI command string to test (e.g., "--version").
    :param contains: Substring expected in the captured stdout.
    :param exitCode: Expected exit code of the CLI command.
    :raises AssertionError: If the exit code or output validation fails.
    """
    print(f"Testing {cli}")

    ret: int = 0
    try:
        ret = cli_entry_point(cli.split())
    except SystemExit as e:
        ret = e.code

    captured = capsys.readouterr()
    assert ret == exitCode, f"Expected exit code {exitCode}, got {ret}."
    assert contains in captured.out, f"Expected output to contain: {contains}"


def test_main_manCore(capsys) -> None:
    """
    Test the `--man` option for core manual page output.

    Verifies that running the CLI with `--man` outputs the manual page and
    exits with code 2.
    """
    CLIcall_parseForStringAndRet(capsys, "--man", "python (pf) mongodb client", 2)


def test_main_version(capsys) -> None:
    """
    Test the `--version` option for version information.

    Verifies that running the CLI with `--version` outputs version info and
    exits with code 1.
    """
    CLIcall_parseForStringAndRet(capsys, "--version", "Version:", 1)


def test_imp_help(capsys) -> None:
    """
    Test the help message for the `fs imp` subcommand.

    This test ensures that running `fs imp --help` outputs the expected
    help text for the `imp` subcommand and exits successfully.

    :raises SystemExit: Ensures the subcommand exits with code 0.
    """
    CLIcall_parseForStringAndRet(capsys, "fs imp --help", "import", 0)



from pfmisc import Colors as C
import click
import pudb
from pfmongo import env
from pfmongo.commands.dbop import connect, showAll, deleteDB


NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN


@click.group(
    cls=env.CustomGroup,
    help=f"""
commands suitable for database focus: {GR}show, connect, deletedb{NC}

This command group provides mongo "database" level commands, allowing
connection to a new database, deletion of an existing database, and
listing all existing databases.

""",
)
def database():
    pass


database.add_command(connect.connect)
database.add_command(showAll.showAll)
database.add_command(deleteDB.deleteDB)



import click
from pfmongo import driver, env
from argparse import Namespace
from pfmisc import Colors as C
from pfmongo.models import responseModel, fsModel
from pathlib import Path
import copy
import ast
import pudb
import asyncio

from pfmongo.commands.document import showAll as doc
from pfmongo.commands.state import showAll as state
import pfmongo.commands.smash as smash

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
PL = C.PURPLE
YL = C.YELLOW


def options_add(options: Namespace) -> Namespace:
    localoptions: Namespace = copy.deepcopy(options)
    localoptions.beQuiet = True
    return localoptions


async def pwd_level(options: Namespace) -> str:
    path: Path = await smash.cwd(options)
    match len(path.parts):
        case 1:
            return "root"
        case 2:
            return "database"
        case 3:
            return "collection"
        case _:
            return "unknown"


def dir_level(dir: fsModel.cdResponse) -> str:
    match len(dir.path.parts):
        case 1:
            return "root"
        case 2:
            return "database"
        case 3:
            return "collection"
        case _:
            return "unknown"


async def pwd_do(options: Namespace) -> fsModel.cdResponse:
    pathResp = fsModel.cdResponse()
    resp: responseModel.mongodbResponse = await state.showAll_asModel(options)
    if resp.message == "/":
        pathResp.path = Path(resp.message)
    else:
        pathResp.path = Path("/") / Path(resp.message)
    pathResp.status = True
    return pathResp


async def pwd_asInt(options) -> int:
    pathResp: fsModel.cdResponse = await pwd_do(options)
    print(pathResp.path)
    return pathResp.code


def sync_pwd_asInt(options: Namespace) -> int:
    return asyncio.run(pwd_asInt(options))


@click.command(
    cls=env.CustomCommand,
    help=f"""
print working {YL}directory{NC}

SYNOPSIS
{CY}pwd{NC}

DESC
This command simply returns the current "working directory".

""",
)
@click.pass_context
def pwd(ctx: click.Context) -> int:
    # pudb.set_trace()
    return sync_pwd_asInt(options_add(ctx.obj["options"]))



import click
from argparse import Namespace
from pfmongo import env, driver
from pfmisc import Colors as C

from pfmongo.commands.clop import connect as collection
from pfmongo.models import responseModel
import pudb
import copy
import asyncio

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
PL = C.PURPLE
YL = C.YELLOW


def options_add(collection: str, options: Namespace) -> Namespace:
    localoptions: Namespace = copy.deepcopy(options)
    localoptions.do = "deleteCollection"
    localoptions.argument = collection
    return localoptions


def col_connectToTarget(options: Namespace) -> str:
    currentCol: str = env.collectionName_get(options)
    if currentCol != options.argument:
        options.do = "connectCollection"
        collection.sync_connectTo_asInt(options)
    return options.argument


def collectiondel_setup(options: Namespace) -> int:
    if env.env_failCheck(options):
        return 100
    col_connectToTarget(options)
    options.do = "deleteCollection"
    return 0


async def collectiondel_asInt(options: Namespace) -> int:
    fail: int = 0
    if fail := collectiondel_setup(options):
        return fail
    return await driver.run_intReturn(options)


async def collectiondel_asModel(options: Namespace) -> responseModel.mongodbResponse:
    model: responseModel.mongodbResponse = responseModel.mongodbResponse()
    fail: int = 0
    if fail := collectiondel_setup(options):
        model.message = "env failure"
        return model
    return await driver.run_modelReturn(options)


def sync_collectiondel_asInt(options: Namespace) -> int:
    return asyncio.run(collectiondel_asInt(options))


def sync_collectiondel_asModel(options: Namespace) -> responseModel.mongodbResponse:
    return asyncio.run(collectiondel_asModel(options))


@click.command(
    cls=env.CustomCommand,
    help=f"""
delete an entire {PL}COLLECTION{NC}

SYNOPSIS
{CY}deletecol {YL}<COLLECTION>{NC}

DESC
This subcommand removes an entire {YL}COLLECTION{NC} immediately.
Use with care! The system does not ask for confirmation!
""",
)
@click.argument("collection", required=True)
@click.pass_context
def deleteCol(ctx: click.Context, collection: str) -> int:
    # pudb.set_trace()
    # options:Namespace   = ctx.obj['options']
    return sync_collectiondel_asInt(options_add(collection, ctx.obj["options"]))



from    pfmongo     import  env
from    pfmisc      import  Colors  as C
import  click
import  pudb

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN
YL  = C.YELLOW

from    pfmongo.commands.stateop    import showAll

@click.group(cls = env.CustomGroup, help=f"""
internal state commands

This command group provides commands operating on internal state.
For the most part, this is simply a {CY}showall{NC} that shows the internal
state (notably the current database and collection of focus).

More information can be returned by calling {GR}pfmongo{NC} with a
{YL}--detailedOutput{NC} flag.

""")
def state():
    pass

state.add_command(showAll.showAll)




import click
from pfmongo import driver, env
from argparse import Namespace
from pfmisc import Colors as C
from pfmongo.models import responseModel, fsModel
from pathlib import Path
import copy
import ast
import pudb
import asyncio

from pfmongo.commands.document import showAll as doc
from pfmongo.commands.state import showAll as state
import pfmongo.commands.smash as smash
import pfmongo.commands.fop.cd as cd


NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
PL = C.PURPLE
YL = C.YELLOW


async def options_add(options: Namespace) -> Namespace:
    localoptions: Namespace = copy.deepcopy(options)
    localoptions.beQuiet = True
    cwd: responseModel.mongodbResponse = await state.showAll_asModel(localoptions)
    localoptions.cwd = "/" + cwd.message
    return localoptions


def cd_options(options: Namespace) -> Namespace:
    mkdir: bool = True
    return cd.options_add(options.cwd, options, mkdir)


async def prompt_do(options: Namespace) -> fsModel.cdResponse:
    # pudb.set_trace()
    promptResp: fsModel.cdResponse = await cd.changeDirectory(cd_options(options))
    (db, collection) = cd.path_to_dbCol(cd_options(options))
    if db:
        promptResp.message = "/"
        match promptResp.state["database"]:
            case "void":
                promptResp.message += f"{YL}{db}{NC}"
            case "exists":
                promptResp.message += f"{GR}{db}{NC}"
    if collection:
        promptResp.message += "/"
        match promptResp.state["collection"]:
            case "void":
                promptResp.message += f"{YL}{collection}{NC}"
            case "exists":
                promptResp.message += f"{GR}{collection}{NC}"
    return promptResp


async def prompt_asModel(options: Namespace) -> fsModel.cdResponse:
    return await prompt_do(await options_add(options))


async def prompt_asInt(options: Namespace) -> int:
    pathResp: fsModel.cdResponse = await prompt_do(await options_add(options))
    print(pathResp.message)
    return pathResp.code


def sync_prompt_asInt(options: Namespace) -> int:
    return asyncio.run(prompt_asInt(options))


def sync_prompt_asModel(options: Namespace) -> fsModel.cdResponse:
    return asyncio.run(prompt_asModel(options))


@click.command(
    cls=env.CustomCommand,
    help=f"""
return a {YL}color {CY}aware{NC} prompt

SYNOPSIS
{CY}prompt{NC}

DESC
This command returns a prompt where the color of a directory indicates its
existence in the monogo server: {YL}yellow{NC} indicates the directory is connected,
but empty; {GR}green{NC} indicates connected and has contents.

Only mongodb "directories" that have contents are listable. What this means
in terms of file system analogues is that in a file system a {YL}mkdir{NC}
command actually creates something that at first is empty. In monogo, a
{YL}database{NC} or {YL}collection{NC} is only "really real" once it is not empty.

""",
)
@click.pass_context
def prompt(ctx: click.Context) -> int:
    # pudb.set_trace()
    return sync_prompt_asInt(ctx.obj["options"])



import click
from pfmongo import driver, env
from argparse import Namespace
from pfmisc import Colors as C
from pfmongo.models import responseModel
from pathlib import Path
import pudb
import copy

import pfmongo.commands.fop.cd as cd

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
PL = C.PURPLE
YL = C.YELLOW


@click.command(
    cls=env.CustomCommand,
    help=f"""
make-and-cd into a {YL}directory{NC}

SYNOPSIS
{CY}mkcd {YL}<path>{NC}

DESC
The {CY}mkcd{NC} is analogous to a combined "mkdir" and "cd" command. It
"creates a new directory" {YL}path{NC} within a mongodb and performs a
{CY}cd{NC} into that {YL}path{NC}.

In reality this is simply an alias to {CY}cd {YL}--create <path>{NC}

""",
)
@click.pass_context
@click.argument("path", required=False)
def mkcd(ctx: click.Context, path: str) -> int:
    # pudb.set_trace()
    mkdir: bool = True
    return cd.sync_changeDirectory(cd.options_add(path, ctx.obj["options"], mkdir)).code



import sys
import re
from setuptools import setup

_version_re = re.compile(r"(?<=^__version__ = (\"|'))(.+)(?=\"|')")


def get_version(rel_path: str) -> str:
    """
    Searches for the ``__version__ = `` line in a source code file.

    https://packaging.python.org/en/latest/guides/single-sourcing-package-version/
    """
    with open(rel_path, "r") as f:
        matches = map(_version_re.search, f)
        filtered = filter(lambda m: m is not None, matches)
        version = next(filtered, None)
        if version is None:
            raise RuntimeError(f"Could not find __version__ in {rel_path}")
        return version.group(0)


requirements = []
with open("requirements.txt") as f:
    requirements = [
        line.strip()
        for line in f.readlines()
        if line.strip() and not line.strip().startswith("#")
    ]

# Make sure we are running python3.5+
if 10 * sys.version_info[0] + sys.version_info[1] < 35:
    sys.exit("Sorry, only Python 3.5+ is supported.")


def readme():
    with open("README.rst") as f:
        return f.read()


setup(
    name="pfmongo",
    version=get_version("pfmongo/__init__.py"),
    description='A mongodb (portable) "pf" client',
    long_description=readme(),
    author="FNNDSC",
    author_email="dev@babymri.org",
    url="https://github.com/FNNDSC/pfmongo",
    packages=[
        "pfmongo",
        "pfmongo/commands",
        "pfmongo/commands/fop",
        "pfmongo/commands/dbop",
        "pfmongo/commands/clop",
        "pfmongo/commands/docop",
        "pfmongo/commands/stateop",
        "pfmongo/commands/slib",
        "pfmongo/db",
        "pfmongo/config",
        "pfmongo/models",
    ],
    install_requires=requirements,
    data_files=[
        ("", ["requirements.txt"]),
    ],
    entry_points={
        "console_scripts": [
            "pfmongo = pfmongo.__main__:main",
            "smashes = pfmongo.smashes:main",
        ]
    },
    license="MIT",
    zip_safe=False,
)



from pfmongo import pfmongo
from pfmongo.__main__ import main
import os
import pudb
from pfmongo.commands.dbop import connect as database, deleteDB
from pfmongo.commands.clop import connect as collection
from pfmongo.commands.docop import add, get
from pfmongo.commands.document import delete as doc
from pfmongo.models import responseModel
from pfmongo.config import settings
import json
import pytest
import asyncio

os.environ["XDG_CONFIG_HOME"] = "/tmp"


@pytest.fixture(autouse=True)
async def cleanup():
    """
    Cleanup databases after each test function in this file.
    """
    await DB_delete()
    yield
    await DB_delete()


@pytest.mark.asyncio
async def DB_connect(DB: str = "testDB") -> int:
    """
    Connect to the specified MongoDB database.
    """
    return await database.connectTo_asInt(
        database.options_add(DB, pfmongo.options_initialize())
    )


@pytest.mark.asyncio
async def collection_connect(col: str = "testCollection") -> int:
    """
    Connect to the specified MongoDB collection.
    """
    return await collection.connectTo_asInt(
        collection.options_add(col, pfmongo.options_initialize())
    )


@pytest.mark.asyncio
async def DB_delete(DB: str = "testDB") -> int:
    """
    Delete the specified MongoDB database.
    """
    return await deleteDB.DBdel_asInt(
        deleteDB.options_add(DB, pfmongo.options_initialize())
    )


@pytest.mark.asyncio
async def test_document_add_asInt() -> None:
    """
    Test adding a document to MongoDB using a file as input.
    """
    await DB_delete()
    await DB_connect()
    await collection_connect()
    retlld: int = await add.documentAdd_asInt(
        add.options_add("examples/lld.json", "lld.json", pfmongo.options_initialize())
    )
    retneuro1: int = await add.documentAdd_asInt(
        add.options_add(
            "examples/neuro1.json", "neuro1.json", pfmongo.options_initialize()
        )
    )
    retneuro2: int = await add.documentAdd_asInt(
        add.options_add(
            "examples/neuro2.json", "neuro2.json", pfmongo.options_initialize()
        )
    )
    retultrasound: int = await add.documentAdd_asInt(
        add.options_add(
            "examples/ultrasound.json", "ultrasound.json", pfmongo.options_initialize()
        )
    )
    assert retlld == 0
    assert retneuro1 == 0
    assert retneuro2 == 0
    assert retultrasound == 0
    await DB_delete()


@pytest.mark.asyncio
async def test_document_add_with_serialized_JSON() -> None:
    """
    Test adding a document to MongoDB using a JSON serialized string as input.
    """
    await DB_delete()
    await DB_connect()
    await collection_connect()

    # JSON serialized string
    json_input = (
        '{"_id": "doc.json", "key1": "value1", "key2": {"nestedKey": "nestedValue"}}'
    )

    # Add the JSON serialized string as a document
    ret: int = await add.documentAdd_asInt(
        add.options_add(json_input, "doc.json", pfmongo.options_initialize())
    )
    assert ret == 0

    # Validate the document exists in the database
    result: responseModel.mongodbResponse = await get.documentGet_asModel(
        get.options_add("doc.json", pfmongo.options_initialize([{"beQuiet": True}]))
    )
    assert result.status
    d_read: dict[str, str] = json.loads(result.message)
    assert d_read["_id"] == "doc.json"
    await DB_delete()


@pytest.mark.asyncio
async def test_duplicate_add_asInt() -> None:
    """
    Test handling duplicate document additions using a file as input.
    """
    await DB_delete()
    await DB_connect()
    await collection_connect()
    retlld: int = await add.documentAdd_asInt(
        add.options_add("examples/lld.json", "lld.json", pfmongo.options_initialize())
    )
    assert retlld == 0
    retlld = await add.documentAdd_asInt(
        add.options_add("examples/lld.json", "lld.json", pfmongo.options_initialize())
    )
    assert retlld == 103
    await DB_delete()


@pytest.mark.asyncio
async def test_duplicateID_add_asModel() -> None:
    """
    Test handling duplicate IDs during document additions.
    """
    await DB_delete()
    await DB_connect()
    await collection_connect()
    retlld: responseModel.mongodbResponse = await add.documentAdd_asModel(
        add.options_add(
            "examples/lld.json",
            "lld.json",
            pfmongo.options_initialize([{"noHashing": True}]),
        )
    )
    retlld = await add.documentAdd_asModel(
        add.options_add(
            "examples/lld.json",
            "lld.json",
            pfmongo.options_initialize([{"noHashing": True}]),
        )
    )
    assert "Could not add" in retlld.message
    assert not retlld.response["status"]
    assert "E11000 duplicate key" in retlld.response["connect"].resp["error"]
    await DB_delete()


@pytest.mark.asyncio
async def test_document_get_asModel() -> None:
    """
    Test retrieving a document as a model.
    """
    await DB_delete()
    await DB_connect()
    await collection_connect()
    load: int = await add.documentAdd_asInt(
        add.options_add("examples/lld.json", "lld.json", pfmongo.options_initialize())
    )
    assert load == 0
    read: responseModel.mongodbResponse = await get.documentGet_asModel(
        get.options_add("lld.json", pfmongo.options_initialize())
    )
    d_read: dict[str, str] = json.loads(read.message)
    assert d_read["_id"] == "lld.json"
    await DB_delete()


@pytest.mark.asyncio
async def test_document_del_asModel() -> None:
    """
    Test deleting document as a model.
    """
    await DB_delete()
    await DB_connect()
    await collection_connect()
    load: int = await add.documentAdd_asInt(
        add.options_add("examples/lld.json", "lld.json", pfmongo.options_initialize())
    )
    assert load == 0
    delete: responseModel.mongodbResponse = await doc.deleteDo_asModel(
        doc.options_add("lld.json", pfmongo.options_initialize())
    )
    assert "Successfully deleted" in delete.message
    assert delete.response["connect"].documentName == "lld.json"
    await DB_delete()


@pytest.mark.asyncio
async def test_deleteTestDB() -> None:
    """
    Test deleting the test MongoDB database.
    """
    ret: int = await deleteDB.DBdel_asInt(
        deleteDB.options_add("testDB", pfmongo.options_initialize())
    )
    assert ret == 0


if __name__ == "__main__":
    print("Test document operations")
    pudb.set_trace()
    asyncio.run(test_document_del_asModel())



import click
import click
import pudb
from pfmongo import driver, env
from argparse import Namespace
from pfmongo.models import responseModel
from pfmisc import Colors as C
import copy
from pathlib import Path
from pfmongo.models import fsModel, responseModel

import pfmongo.commands.smash as smash
import pfmongo.commands.fop.cd as cd
import pfmongo.commands.docop.get as get
import asyncio

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
PL = C.PURPLE
YL = C.YELLOW


def options_add(file: str, options: Namespace) -> Namespace:
    localoptions: Namespace = copy.deepcopy(options)
    localoptions.do = "cat"
    localoptions.file = Path(file)
    localoptions.beQuiet = True
    return localoptions


async def path_process(options: Namespace) -> fsModel.cdResponse:
    dir: Path = options.file.parent
    return await cd.changeDirectory(cd.options_add(str(dir), options))


async def cat_do(options: Namespace) -> int:
    cwd: Path = await smash.cwd(options)
    fileDir: fsModel.cdResponse = await path_process(options)
    if not fileDir.status:
        print("directory {fileDir.path} does not seem to exist")
    getResponse: responseModel.mongodbResponse = await get.documentGet_asModel(
        get.options_add(str(options.file.name), options)
    )
    print(getResponse.message)
    await cd.changeDirectory(cd.options_add(str(cwd), options))
    return 0


def sync_cat_do(options: Namespace) -> int:
    return asyncio.run(cat_do(options))


@click.command(
    cls=env.CustomCommand,
    help=f"""
show {YL}document{NC} contents

SYNOPSIS
{CY}cat {YL}<file>{NC}

DESC
Read a {YL}file{NC} to stdout. Note that the {YL}<file>{NC} can consist of a
{YL}<path>{NC} prefix specifier denoting the {YL}database{NC} and {YL}collection{NC}
to use.

""",
)
@click.pass_context
@click.argument("path", required=False)
def cat(ctx: click.Context, path: str) -> int:
    # pudb.set_trace()
    return sync_cat_do(options_add(path, ctx.obj["options"]))
